// *****************************************************************************
//
//     Copyright (c) 2007, Pleora Technologies Inc., All rights reserved.
//
// *****************************************************************************

#include "stdafx.h"

#include "PvSimpleUISample.h"
#include "PvSimpleUISampleDlg.h"
#include "AlarmSetupDlg.h"
#include "AreaSetupDlg.h"
#include "AboutDlg.h"
#include "LogDlg.h"
#include "CalibDlg.h"
#include "temperature.hpp"
#include "EditObjParsDlg.h"

#include <string.h>
#include <math.h>

#include <PvGenStateStack.h>

#define ASY_SAFEGUARD 1.0002
#define EXP_SAFEGUARD 709.78

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*----------------------------------------------------------------------------
      Function: quicklog

     Arguments: Value to take the e-logarithm of. Must be > 0

       Returns: ln(var) IN SINGLE PRECISION
                The accuracy is about 5-6 digits between 0.995 to 1.005.
                Extremely close to 1.00 it is just 3 digits.
                It is a lot better outside this region (up to 9-10 digits).
----------------------------------------------------------------------------*/
static double quicklog(double dbl_val)
{
#define sqr2div2 0.7071067811865476
#define ln2 0.6931471805599453
#define C1 2.88539129   
#define C3 0.961470632
#define C5 0.598978649

#ifdef _WIN32
/* Little endian machine assumed */
 #define DBL_2EXP_ADDR(dbl) ((unsigned short int*)&dbl+3)
 #define TRUE_DBL_2EXP(dbl) (((*DBL_2EXP_ADDR(dbl)&0x7ff0)>>4)-0x3ff)
 #define SET_DBL_2EXP(dbl) *DBL_2EXP_ADDR(dbl)&=~0x7ff0,\
                          *DBL_2EXP_ADDR(dbl)|= 0x3fe0
#else
/* Big endian machine assumed */
 #define DBL_2EXP_ADDR(dbl) ((unsigned long int*)&dbl)
 #define TRUE_DBL_2EXP(dbl) (((*DBL_2EXP_ADDR(dbl)&0x7ff00000)>>20)-0x3ff)
 #define SET_DBL_2EXP(dbl) *DBL_2EXP_ADDR(dbl)&=~0x7ff00000,\
                          *DBL_2EXP_ADDR(dbl)|= 0x3fe00000
#endif


   double Z, Z2;
   int times_2;
   
   times_2 = TRUE_DBL_2EXP(dbl_val)+1;
   SET_DBL_2EXP(dbl_val); /* to -1 */

   Z = (dbl_val - sqr2div2)/(dbl_val + sqr2div2);
   Z2 = Z*Z;
   return (times_2+(((C5*Z2+C3)*Z2+C1)*Z-0.5))*ln2;

#undef sqr2div2
#undef ln2
#undef C1
#undef C3
#undef C5
#undef DBL_2EXP_ADDR
#undef TRUE_DBL_2EXP
#undef SET_DBL_2EXP
}

BEGIN_MESSAGE_MAP(PvSimpleUISampleDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_SIZE()
	ON_WM_GETMINMAXINFO()
	ON_BN_CLICKED(IDC_DEVICE_BUTTON, &PvSimpleUISampleDlg::OnBnClickedDeviceButton)
	ON_BN_CLICKED(IDC_LINK_BUTTON, &PvSimpleUISampleDlg::OnBnClickedLinkButton)
	ON_BN_CLICKED(IDC_STREAMPARAMS_BUTTON, &PvSimpleUISampleDlg::OnBnClickedStreamparamsButton)
	ON_BN_CLICKED(IDC_CONNECT_BUTTON, &PvSimpleUISampleDlg::OnBnClickedConnectButton)
	ON_BN_CLICKED(IDC_DISCONNECT_BUTTON, &PvSimpleUISampleDlg::OnBnClickedDisconnectButton)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_START, &PvSimpleUISampleDlg::OnBnClickedStart)
	ON_BN_CLICKED(IDC_STOP, &PvSimpleUISampleDlg::OnBnClickedStop)
	ON_BN_CLICKED(IDC_DEVICEEVENTS_BUTTON, &PvSimpleUISampleDlg::OnBnClickedDeviceEvents)
	ON_WM_MOVE()
    ON_WM_CTLCOLOR()
	ON_CBN_SELCHANGE(IDC_PALETTE, &PvSimpleUISampleDlg::OnCbnSelchangePalette)
	ON_BN_CLICKED(IDC_AUTOFOCUS, &PvSimpleUISampleDlg::OnBnClickedAutofocus)
	ON_BN_CLICKED(IDC_SAVE_IMAGE, &PvSimpleUISampleDlg::OnBnClickedSaveImage)
	ON_CBN_SELCHANGE(IDC_FRATE, &PvSimpleUISampleDlg::OnCbnSelchangeFrate)
	ON_CBN_SELCHANGE(IDC_PIXEL_FORMAT, &PvSimpleUISampleDlg::OnCbnSelchangePixelFormat)
	ON_CBN_SELCHANGE(IDC_RANGE, &PvSimpleUISampleDlg::OnCbnSelchangeRange)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_AUTOADJ, &PvSimpleUISampleDlg::OnBnClickedAutoadj)
	ON_BN_CLICKED(IDC_ONESHOTADJ, &PvSimpleUISampleDlg::OnBnClickedOneshotadj)
	ON_EN_KILLFOCUS(IDC_SCLO, &PvSimpleUISampleDlg::OnEnKillfocusSclo)
	ON_EN_KILLFOCUS(IDC_SCHI, &PvSimpleUISampleDlg::OnEnKillfocusSchi)
	ON_BN_CLICKED(IDC_NUC, &PvSimpleUISampleDlg::OnBnClickedNuc)
	ON_BN_CLICKED(IDC_ALARM_SETUP, &PvSimpleUISampleDlg::OnBnClickedAlarmSetup)
	ON_BN_CLICKED(IDC_AREA_SETUP, &PvSimpleUISampleDlg::OnBnClickedAreaSetup)
	ON_BN_CLICKED(IDC_SAVE_BMP, &PvSimpleUISampleDlg::OnBnClickedSaveBmp)
	ON_BN_CLICKED(IDC_ABOUT, &PvSimpleUISampleDlg::OnBnClickedAbout)
	ON_BN_CLICKED(IDC_LOG, &PvSimpleUISampleDlg::OnBnClickedLog)
    ON_BN_CLICKED(IDC_CALIBRATE, &PvSimpleUISampleDlg::OnBnClickedCalibrate)
    ON_MESSAGE(WM_ACQUISITIONSTATECHANGED, &PvSimpleUISampleDlg::OnAcquisitionStateChanged)
    ON_CBN_SELCHANGE(IDC_COMBO1, &PvSimpleUISampleDlg::OnCbnSelchangePresentation)
    ON_BN_CLICKED(IDC_SAVE_RAW, &PvSimpleUISampleDlg::OnBnClickedSaveRaw)
    ON_BN_CLICKED(IDC_OBJPARS, &PvSimpleUISampleDlg::OnBnClickedObjpars)
    ON_MESSAGE(WM_LINKDISCONNECTED, &PvSimpleUISampleDlg::OnLinkDisconnected)
    ON_MESSAGE(WM_LINKRECONNECTED, &PvSimpleUISampleDlg::OnLinkReconnected)
    ON_MESSAGE(WM_STOPSTREAMING, OnStopStreaming)
    ON_MESSAGE(WM_DISCONNECT, OnDisconnect)
    ON_MESSAGE(WM_STREAM, OnStream)
    ON_BN_CLICKED(IDC_SAVE_SEQ, &PvSimpleUISampleDlg::OnBnClickedSaveSeq)
END_MESSAGE_MAP()


// =============================================================================
PvSimpleUISampleDlg::PvSimpleUISampleDlg( CWnd* pParent /* =NULL */ )
	: CDialog( PvSimpleUISampleDlg::IDD, pParent )
	, mNeedInit( TRUE )
	, mDeviceWnd( NULL )
	, mCommunicationWnd( NULL )
	, mStreamParametersWnd( NULL )
	, mThreadDisplay( NULL )
    , mAcquiringImages( false )
    , mAcquisitionStateManager( NULL )
    , mRecovery( FALSE )
	, m_fScaleLo(0)
	, m_fScaleHi(0)
    , m_uiSeqFrames(0)
{
	mPipeline = NULL;
	mStream = NULL;
	mDevice = NULL;
	m_hIcon = AfxGetApp()->LoadIcon( IDR_MAINFRAME );
	m_Width = 320;
	m_Height = 240;
	m_CamType = CAM_TYPE_A3XX;
    m_WindowingMode = 0; // Assume Off
	m_Timer = 0;
	m_bAlarmActive = false;
	m_iAlarmFunc = ALARM_FUNC_SPOT;
	m_iAlarmResult = ALARM_RES_VALUE;
	m_iAlarmCondition = ALARM_COND_ABOVE;
	m_PulseTime = 2000;
	m_DigOut = 0; // No digital output port action on alarm
	m_dblAlarmValue = 25.0f;
	m_bFlash = FALSE;
	m_bLogging = false;
	m_iLogCount = 0;
    m_iLogInterval = 120;
    m_bMeasCapable = false;
    m_bTLUTCapable = false;
    m_bFlyingSpotActive = false;
    m_bAreaActive = true;
    m_bFast = false; // Used for ATAU
	m_bFrameRateControlEnabled = false;
    m_bSingleRange = false; // We will assume that TAU camera has two ranges
    m_iPresentation = PRES_SIGNAL;
    m_WT = 573.0; // Default value high gain mode
    m_W4WT = 1542.0;
    m_bFlashTS = GetTickCount();
    m_uiSeqFrames = 10;
    m_CalcCheckNeeded = 0;
    m_bRecording = false;

    m_AtmTemp = m_ExtOptTemp = m_AmbTemp = 293.15f;
    m_ExtOptTransm = 1.0f; // default
    // For TAU cameras we need GUI code for editing these values
    m_ObjectDistance = 2.0; // Default
    m_RelHum = 0.5; // Default
    m_Emissivity = 1.0; // Default

    m_J1 = 1;
    m_J0 = 0;
    // Initiate spectral response
    m_X = 1.9;
    m_alpha1 = 0.006569;
    m_beta1 = -0.002276;
    m_alpha2 = 0.01262;
    m_beta2 = -0.00667;
}
// =============================================================================
PvSimpleUISampleDlg::~PvSimpleUISampleDlg()
{
    if (mAcquisitionStateManager != NULL)
    {
        delete mAcquisitionStateManager;
        mAcquisitionStateManager = NULL;
    }
}


// =============================================================================
void PvSimpleUISampleDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_START, mPlayButton);
    DDX_Control(pDX, IDC_STOP, mStopButton);
    DDX_Control(pDX, IDC_IP_EDIT, mIPEdit);
    DDX_Control(pDX, IDC_MAC_EDIT, mMACEdit);
    DDX_Control(pDX, IDC_MODEL_EDIT, mModelEdit);
    DDX_Control(pDX, IDC_MANUFACTURER_EDIT, mManufacturerEdit);
    DDX_Control(pDX, IDC_NAME_EDIT, mNameEdit);
    DDX_Control(pDX, IDC_PALETTE, m_ctrlPalette);
    DDX_Control(pDX, IDC_AUTOFOCUS, m_btnAutoFocus);
    DDX_Control(pDX, IDC_SAVE_IMAGE, m_btnSaveImage);
    DDX_Control(pDX, IDC_FRATE, m_ctrlFrameRate);
    DDX_Control(pDX, IDC_PIXEL_FORMAT, m_ctrlIRFormat);
    DDX_Control(pDX, IDC_RANGE, m_ctrlRange);
    DDX_Control(pDX, IDC_SPOT, m_ctrlSpot);
    DDX_Control(pDX, IDC_AUTOADJ, m_btnAutoAdjust);
    DDX_Control(pDX, IDC_ONESHOTADJ, m_btnAdjustOnce);
    DDX_Control(pDX, IDC_SCLO, m_ctrlScaleLow);
    DDX_Control(pDX, IDC_SCHI, m_ctrlScaleHigh);
    DDX_Text(pDX, IDC_SCLO, m_fScaleLo);
    DDX_Control(pDX, IDC_UNIT, m_ctrlUnit);
    DDX_Text(pDX, IDC_SCHI, m_fScaleHi);
    DDX_Control(pDX, IDC_NUC, m_btnNUC);
    DDX_Control(pDX, IDC_ALARM_STATE, m_ctrlAlarmState);
    DDX_Control(pDX, IDC_TRIG_COUNT2, m_ctrlTrigCount2);
    DDX_Control(pDX, IDC_TRIG_COUNT1, m_ctrlTrigCount1);
    DDX_Control(pDX, IDC_AREA_AVG, m_ctrlAreaAvg);
    DDX_Control(pDX, IDC_AREA_MIN, m_ctrlAreaMin);
    DDX_Control(pDX, IDC_AREA_MAX, m_ctrlAreaMax);
    DDX_Control(pDX, IDC_AREA_SETUP, m_ctrlAreaSetup);
    DDX_Control(pDX, IDC_SAVE_BMP, m_btnSaveBMP);
    DDX_Control(pDX, IDC_DISPLAYPOS, m_btnDisplay);
    DDX_Control(pDX, IDC_LOG, m_btnLog);
    DDX_Control(pDX, IDC_CENTER_SPOT, m_ctrlCenterSpot);
    DDX_Control(pDX, IDC_CALIBRATE, m_ctrlCalibrate);
    DDX_Control(pDX, IDC_STATUS, m_ctrlStatus);
    DDX_Control(pDX, IDC_COMBO1, m_ctrlPresentation);
    DDX_Control(pDX, IDC_SAVE_RAW, m_btnSaveRAW);
    DDX_Control(pDX, IDC_PORT1_STATE, m_ctrlPort1State);
    DDX_Control(pDX, IDC_PORT2_STATE, m_ctrlPort2State);
    DDX_Control(pDX, IDC_OBJPARS, m_ctrlObjPars);
    DDX_Control(pDX, IDC_SAVE_SEQ, m_btnSaveSEQ);
    DDX_Text(pDX, IDC_SEQ_FRAMES, m_uiSeqFrames);
}


// =============================================================================
BOOL PvSimpleUISampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetWindowText( _T( "FLIR GEV DEMO 1.8" ) );

	GetClientRect( mCrt );
	mNeedInit = FALSE;

    InitializeCriticalSection( &m_csPort );

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	CRect lDisplayRect;
	GetDlgItem( IDC_DISPLAYPOS )->GetClientRect( &lDisplayRect );
	GetDlgItem( IDC_DISPLAYPOS )->ClientToScreen( &lDisplayRect );
	ScreenToClient( &lDisplayRect );
    mDisplay.Create( GetSafeHwnd(), 10000 );
	mDisplay.SetPosition( lDisplayRect.left, lDisplayRect.top, lDisplayRect.Width(), lDisplayRect.Height() );
	mDisplay.SetBackgroundColor( 0x80, 0x80, 0x80 );

	m_ctrlPalette.SetCurSel(0);
	m_ctrlPalette.EnableWindow(FALSE);
	m_ctrlAreaSetup.EnableWindow(FALSE);
	m_btnAutoAdjust.SetCheck(BST_CHECKED);
	m_btnAdjustOnce.EnableWindow(FALSE);
	m_btnSaveImage.EnableWindow(FALSE);
	m_btnSaveBMP.EnableWindow(FALSE);
	m_btnSaveRAW.EnableWindow(FALSE);
	m_btnSaveSEQ.EnableWindow(FALSE);
	m_btnLog.EnableWindow(FALSE);
	m_ctrlScaleLow.EnableWindow(FALSE);
	m_ctrlScaleHigh.EnableWindow(FALSE);
    m_ctrlPresentation.EnableWindow(FALSE);
	m_bAutoAdjust = true;
	m_bAdjustOnce = false;

	m_ctrlAlarmState.Attach(IDC_ALARM_STATE, this);
	m_ctrlAlarmState.SetBGColor(WHITE, TRUE);

	m_btnSaveSEQ.Attach(IDC_SAVE_SEQ, this);
	m_btnSaveSEQ.SetBGColor(WHITE, TRUE);

    EnableInterface();
	mbWorkaround =false;

    return TRUE;  // return TRUE  unless you set the focus to a control
}


// =============================================================================
void PvSimpleUISampleDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}


// =============================================================================
HCURSOR PvSimpleUISampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


// =============================================================================
void PvSimpleUISampleDlg::OnClose()
{
	// Make sure we cleanup before we leave
	Disconnect();

	CDialog::OnClose();
}

void PvSimpleUISampleDlg::OnAcquisitionStateChanged(PvDevice* aDevice, 
                                                    PvStream* aStream,
                                                    uint32_t aSource,
                                                    PvAcquisitionState aState)
{
    PostMessage( WM_ACQUISITIONSTATECHANGED );
}

LRESULT PvSimpleUISampleDlg::OnAcquisitionStateChanged(WPARAM wParam, LPARAM lParam)
{
    EnableInterface();
    return 0;
}


// =============================================================================
void PvSimpleUISampleDlg::OnBnClickedConnectButton()
{
	// create a device finder wnd and open the select device dialog
	PvDeviceFinderWnd lFinder;
	lFinder.SetTitle( _T( "GEV Device Selection" ) );

    PvResult lResult = lFinder.ShowModal();

    if ( lResult.GetCode() == PvResult::Code::ABORTED )
    {
        MessageBox( _T( "Invalid selection. Please select a GEV Device connected through an eBUS network card configuration." ), _T("PvSimpleUISample") );
        return;
    }

    if ( !lResult.IsOK() || ( lFinder.GetSelected() == NULL ) )
	{
		return;
	}

	CWaitCursor lCursor;

	UpdateWindow();

    const PvDeviceInfo* lDeviceInfo = lFinder.GetSelected();

	if ( lDeviceInfo != NULL )
	{
		Connect( lDeviceInfo );
	}
	else
	{
		MessageBox( _T( "No device selected." ), _T( "Error" ), MB_OK | MB_ICONINFORMATION );
		return;
	}
}


// =============================================================================
void PvSimpleUISampleDlg::OnBnClickedDisconnectButton()
{
	CWaitCursor lCursor;

	Disconnect();
}


// =============================================================================
void PvSimpleUISampleDlg::OnSize(UINT nType, int cx, int cy)
{
	CDialog::OnSize(nType, cx, cy);

	CRect lR;
	this->GetWindowRect( &lR );
	TRACE( "%i %i\n", lR.Width(), lR.Height() );

	if ( mNeedInit || cx == 0 || cy == 0 )
	{
		return;
	}

	int dx = cx - mCrt.Width();
	int dy = cy - mCrt.Height();

    GetClientRect(&mCrt);

	CRect r1;

	HDWP hDWP = ::BeginDeferWindowPos(20);

	//
	// Bottom right
	//
	int lBottomRight[] =
	{
        IDC_STATUS,
		IDC_ALARM_GROUP, IDC_ALARM_SETUP, IDC_ALARM_STATE, IDC_CAMCONTROL_GROUP,
		IDC_AUTOFOCUS, IDC_NUC, IDC_PIXEL_FORMAT, IDC_RANGE, IDC_FRATE, IDC_CALIBRATE, IDC_OBJPARS,
		IDC_PIXEL_FORMAT_TEXT, IDC_RANGE_TEXT, IDC_FRATE_TEXT,
		IDC_SPOT_GROUP, IDC_SPOT, IDC_SPOT_TEXT, IDC_CENTER_SPOT, IDC_CENTER_SPOT_TEXT,
		IDC_AREA_GROUP, IDC_AREA_SETUP, IDC_AREA_AVG, IDC_AREA_MIN, IDC_AREA_MAX,
		IDC_AREA_AVG_TEXT, IDC_AREA_MIN_TEXT, IDC_AREA_MAX_TEXT
	};

	for ( int i = 0; i < sizeof( lBottomRight ) / sizeof ( lBottomRight[ 0 ] ); i++ )
	{
		CWnd *lChild = GetDlgItem( lBottomRight[ i ] );
		if ( lChild != NULL )
		{
			lChild->GetWindowRect(&r1);
			ScreenToClient(&r1); 
			r1.bottom += dy;
			r1.top += dy;
			::DeferWindowPos(
				hDWP, lChild->m_hWnd, NULL, 
				r1.left, r1.top, r1.Width(), r1.Height(),
				SWP_NOACTIVATE|SWP_NOZORDER);
		}
	}

	//
	// Right, make sure the display and group box fill the right part
	// of our window
	//

    int lDisplayGroup[] =
	{
		IDC_DISPLAY_GROUP, IDC_DISPLAYPOS
	};

	for ( int i = 0; i < sizeof( lDisplayGroup ) / sizeof ( lDisplayGroup[ 0 ] ); i++ )
	{
		CWnd *lChild = GetDlgItem( lDisplayGroup[ i ] );
		if ( lChild != NULL )
		{
			lChild->GetWindowRect(&r1);
			ScreenToClient(&r1); 
			r1.bottom += dy;
            r1.right += dx;
			::DeferWindowPos(
				hDWP, lChild->m_hWnd, NULL, 
				r1.left, r1.top, r1.Width(), r1.Height(),
				SWP_NOACTIVATE|SWP_NOZORDER);

			if ( lDisplayGroup[i] == IDC_DISPLAYPOS )
			{
				mDisplay.SetPosition( 
					r1.left, r1.top, 
					r1.Width(), r1.Height() );
			}
		}
	}

	::EndDeferWindowPos(hDWP);
}


// =============================================================================
void PvSimpleUISampleDlg::OnGetMinMaxInfo( MINMAXINFO *lpMMI )
{
	lpMMI->ptMinTrackSize.x = 800;
	lpMMI->ptMinTrackSize.y = 497;
}


// =============================================================================
void PvSimpleUISampleDlg::EnableInterface()
{
	// This method can be called really early or late when the window is not created
	if ( GetSafeHwnd() == 0 )
	{
		return;
	}

	bool lConnected = ( mDevice != NULL ) && mDevice->IsConnected();

	GetDlgItem( IDC_CONNECT_BUTTON )->EnableWindow( !lConnected );
	GetDlgItem( IDC_DISCONNECT_BUTTON )->EnableWindow( lConnected );

	GetDlgItem( IDC_DEVICE_BUTTON )->EnableWindow( lConnected );
	GetDlgItem( IDC_STREAMPARAMS_BUTTON )->EnableWindow( lConnected );
	GetDlgItem( IDC_COMMUNICATION_BUTTON )->EnableWindow( lConnected  ); 

	mPlayButton.EnableWindow( lConnected );
	m_btnAutoFocus.EnableWindow( (m_CamType != CAM_TYPE_ATAU) && lConnected );
	m_btnNUC.EnableWindow( lConnected );
	m_ctrlFrameRate.EnableWindow( m_bFrameRateControlEnabled && lConnected );
	m_ctrlIRFormat.EnableWindow( lConnected );
	m_ctrlRange.EnableWindow( lConnected );
	m_ctrlObjPars.EnableWindow( lConnected );
	m_btnAutoAdjust.EnableWindow( lConnected );
	m_btnLog.EnableWindow(lConnected && (m_CamType == CAM_TYPE_ATAU));
	m_ctrlCalibrate.EnableWindow((m_CamType == CAM_TYPE_ATAU) && m_bMeasCapable && lConnected && (m_IRFormat == IRFMT_SIGNAL) );
    if (m_CamType == CAM_TYPE_ATAU)
        m_ctrlPresentation.EnableWindow( lConnected && m_bMeasCapable );
    else
        m_ctrlPresentation.EnableWindow( lConnected && ((m_IRFormat > IRFMT_SIGNAL) || m_bMeasCapable));

    bool lLocked = false;
    if (mAcquisitionStateManager != NULL)
    {
        lLocked = mAcquisitionStateManager->GetState() == PvAcquisitionStateLocked;
    }

	// If not connected, disable the acquisition mode control. If enabled,
	// it will be managed automatically by events from the GenICam parameters
	mStopButton.EnableWindow( lConnected );
	mPlayButton.EnableWindow( lConnected );
}
// =============================================================================
LRESULT PvSimpleUISampleDlg::OnStopStreaming( WPARAM wParam, LPARAM lParam )
{
    if ( mStream->IsOpen() )
    {
        StopStreaming();
        mStream->Close();
    }

    return 0;
}
// =============================================================================
LRESULT PvSimpleUISampleDlg::OnLinkDisconnected( WPARAM wParam, LPARAM lParam )
{
    mRecovery = TRUE;
    SendMessage( WM_STOPSTREAMING );
    SendMessage( WM_DISCONNECT );
    return 0;
}
// =============================================================================
LRESULT PvSimpleUISampleDlg::OnLinkReconnected( WPARAM wParam, LPARAM lParam )
{
	PvGenEnum       *lPixelFormat = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("PixelFormat"));
    PvGenInteger    *lInteger = mDevice->GetParameters()->GetInteger( "GevCurrentIPAddress" );

    if ( lInteger != NULL )
    {
        int64_t lValue;
        lInteger->GetValue( lValue );

        CString lIP;
        lIP.Format( _T( "%I64i.%I64i.%I64i.%I64i" ),
            ( lValue & 0xFF000000 ) >> 24,
            ( lValue & 0x00FF0000 ) >> 16,
            ( lValue & 0x0000FF00 ) >> 8,
            ( lValue & 0x000000FF ) );

        mIPEdit.SetWindowText( lIP );
    }

    // Important - this will, amongst other, make sure TLParamsLocked is set to 0!
    StopAcquisition();

    if ( lPixelFormat != NULL )
    {
        // This is needed in case the device has been power cycled
        if (m_CamType == CAM_TYPE_ATAU)
        {
            PvGenEnum* lCMOSBitDepth = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("CMOSBitDepth"));
            if (lCMOSBitDepth) 
                lCMOSBitDepth->SetValue(0); // 14 bit

            lPixelFormat->SetValue(0x01100025); // Mono14
        }
	    else
    		lPixelFormat->SetValue(0x01100007); // Mono16
	}

    // Restart streaming
    SendMessage( WM_STREAM, 1 );
    m_ctrlStatus.SetWindowText(_T( "" ));
    mRecovery = FALSE;
    return 0;
}
// =============================================================================
bool PvSimpleUISampleDlg::IsMultiSourceTransmitter( PvDevice *aDevice )
{
	if ( ( aDevice != NULL ) && aDevice->IsConnected() )
	{
		PvGenEnum *lEnum = aDevice->GetParameters()->GetEnum( "SourceSelector" );
		PvGenEnum *lDeviceClass = aDevice->GetParameters()->GetEnum( "GevDeviceClass" );

		if ( ( lEnum != NULL ) && ( lDeviceClass != NULL ) )
		{
			PvString lClass;
			if ( lDeviceClass->GetValue( lClass ).IsOK() )
			{
				return ( lClass == "Transmitter" );
			}
		}
	}

	return false;
}
// =============================================================================
void PvSimpleUISampleDlg::PushSource( PvGenStateStack *aStack )
{
    // If multi source, push the active source on the stack
    if ( IsMultiSourceTransmitter( mDevice ) )
    {
        if ( mAcquisitionStateManager != NULL )
        {
            // Select source (will be reset when lStack goes out of scope)
            aStack->SetEnumValue( "SourceSelector", mAcquisitionStateManager->GetSource() );
        }
    }
}
// =============================================================================
LRESULT PvSimpleUISampleDlg::OnStream( WPARAM wParam, LPARAM lParam )
{
    if ( !mDevice->IsConnected() )
    {
        return 0;
    }
    
    PvGenStateStack lStack( mDevice->GetParameters() );
    PushSource( &lStack );

    int64_t lChannel = 0;
    if ( IsMultiSourceTransmitter( mDevice ) )
    {
        mDevice->GetParameters()->GetIntegerValue( "SourceStreamChannel", lChannel );
    }

    PvGenInteger *lInteger = mDevice->GetParameters()->GetInteger( "GevCurrentIPAddress" );
    ASSERT( lInteger != NULL );

    int64_t lValue;
    lInteger->GetValue( lValue );

    CString lIP;
    lIP.Format( _T( "%I64i.%I64i.%I64i.%I64i" ),
            ( lValue & 0xFF000000 ) >> 24,
            ( lValue & 0x00FF0000 ) >> 16,
            ( lValue & 0x0000FF00 ) >> 8,
            ( lValue & 0x000000FF ) );

    PvResult lResult = PvResult::Code::GENERIC_ERROR;
    // Reopen stream
   	if ( mStream != NULL )
	{
        PvStream::Free( mStream );
        mStream = NULL;
	}

    mStream = PvStream::CreateAndOpen(mConnectionID,  &lResult );

    if ( lResult.IsOK() )
    {
        PvDeviceGEV *lDeviceGEV = static_cast<PvDeviceGEV *>( mDevice );
        PvStreamGEV *lStreamGEV = static_cast<PvStreamGEV *>( mStream );

	    PvString lLocalIpAddress = lStreamGEV->GetLocalIPAddress();
	    uint16_t lLocalPort = lStreamGEV->GetLocalPort();

        // Now that the stream is opened, set the destination on the device
		lDeviceGEV->SetStreamDestination( lLocalIpAddress, lLocalPort );

        StartStreaming();

        if ( wParam != 0 )
        {
           StartAcquisition();
        }
    }

    m_ctrlStatus.SetWindowText(_T( "" ));
	EnableInterface();
    return 0;
}
// =============================================================================
LRESULT PvSimpleUISampleDlg::OnDisconnect( WPARAM wParam, LPARAM lParam )
{
    // Is reconnecting enabled?
    PvGenBoolean *lLinkRecoveryEnabled = mDevice->GetCommunicationParameters()->GetBoolean( "LinkRecoveryEnabled"  );
    ASSERT( lLinkRecoveryEnabled != NULL );

    bool lEnabled = false;
    lLinkRecoveryEnabled->GetValue( lEnabled );

    if ( lEnabled )
    {
        m_ctrlStatus.SetWindowText(_T( "Connection to device lost..." ));

		CloseGenWindow( &mDeviceWnd );
		CloseGenWindow( &mCommunicationWnd );
		CloseGenWindow( &mStreamParametersWnd );
    }
    else
    {
        m_ctrlStatus.SetWindowText(_T( "" ));

        CWaitCursor lCursor;
        Disconnect();
    }

    return 0;
}
// PvDeviceEventSink implementation
// =============================================================================
void PvSimpleUISampleDlg::OnLinkDisconnected( PvDevice *aDevice )
{
    PostMessage( WM_LINKDISCONNECTED );    
}
// PvDeviceEventSink implementation
// =============================================================================
void PvSimpleUISampleDlg::OnLinkReconnected( PvDevice *aDevice )
{
    PostMessage( WM_LINKRECONNECTED );
}
// PvDeviceEventSink implementation
// =============================================================================
void PvSimpleUISampleDlg::OnEvent( PvDevice *aDevice, uint16_t aEventID, uint16_t aChannel, uint16_t aBlockID, 
    uint64_t aTimestamp, const void *aData, uint32_t aDataLength )
{
}

// =============================================================================
void PvSimpleUISampleDlg::Connect( const PvDeviceInfo *aDI )
{
	ASSERT( aDI != NULL );
	if ( aDI == NULL )	
    {
		return;
	}

 	// Just in case we came here still connected...
	Disconnect();

    // Device connection, packet size negociation and stream opening
	PvResult lResult = PvResult::Code::NOT_CONNECTED;

    // Connect device
	mDevice = PvDevice::CreateAndConnect( aDI, &lResult );
    if ( !lResult.IsOK() )
    {
		Disconnect();
		return;
	}

    PvGenParameterArray *lGenDevice = mDevice->GetCommunicationParameters();
    lGenDevice->SetIntegerValue("AnswerTimeout", 1000);
    lGenDevice->SetIntegerValue("CommandRetryCount", 5);

    // Open stream
    if ( aDI->GetType() == PvDeviceInfoTypeGEV )
        mConnectionID = aDI->GetConnectionID(); // Save connection ID
    else if  ( aDI->GetType() == PvDeviceInfoTypeU3V ) 
    {
        const PvDeviceInfoU3V *lDeviceInfo = dynamic_cast<const PvDeviceInfoU3V *>( aDI );

        mConnectionID = lDeviceInfo->GetDeviceGUID();
    }

	mStream = PvStream::CreateAndOpen( mConnectionID, &lResult );
	if ( !lResult.IsOK() )
	{
        Disconnect();
        return;
	}

    if ( aDI->GetType() == PvDeviceInfoTypeGEV )
    {
        PvDeviceGEV *lDeviceGEV = static_cast<PvDeviceGEV *>( mDevice );
        PvStreamGEV *lStreamGEV = static_cast<PvStreamGEV *>( mStream );

	    PvString lLocalIpAddress = lStreamGEV->GetLocalIPAddress();
	    uint16_t lLocalPort = lStreamGEV->GetLocalPort();

		// Perform automatic packet size negotiation
		lDeviceGEV->NegotiatePacketSize();

        // Now that the stream is opened, set the destination on the device
		lDeviceGEV->SetStreamDestination( lLocalIpAddress, lLocalPort );
    }
    else if ( aDI->GetType() == PvDeviceInfoTypeU3V )
    {
        TRACE("U3V device");
    }

    mPipeline = new PvPipeline( mStream );

    mRecovery = FALSE;

    // Register device event sink
    mDevice->RegisterEventSink( this );

    // Register to all events of the parameters in the device's node map
    lGenDevice = mDevice->GetParameters();
    for ( uint32_t i = 0; i < lGenDevice->GetCount(); i++ )
    {
	    lGenDevice->Get( i )->RegisterEventSink( this );
    }

	PvGenInteger *lIPAddressParam = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "GevCurrentIPAddress" ) );
	PvGenInteger *lMACAddressParam = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "GevMACAddress" ) );
	PvGenString *lManufacturer = dynamic_cast<PvGenString *>( lGenDevice->Get( "DeviceVendorName" ) );
	PvGenString *lModelName = dynamic_cast<PvGenString *>( lGenDevice->Get( "DeviceModelName" ) );
	PvGenString *lDeviceVersion = dynamic_cast<PvGenString *>( lGenDevice->Get( "DeviceVersion" ) );
	PvGenString *lNameParam = dynamic_cast<PvGenString *>( lGenDevice->Get( "GevDeviceUserID " ) );
	PvGenString *lManufacturerInfo = dynamic_cast<PvGenString *>( lGenDevice->Get( "DeviceManufacturerInfo" ) );
	PvGenInteger *lWidth = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "Width" ) );
	PvGenInteger *lHeight = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "Height" ) );
	PvGenEnum *lFrameRate = dynamic_cast<PvGenEnum *>( lGenDevice->Get( "IRFrameRate" ) );
	PvGenEnum *lIRFormat = dynamic_cast<PvGenEnum *>( lGenDevice->Get( "IRFormat" ) );
	PvGenEnum* lPixelFormat = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("PixelFormat"));
	PvGenInteger *lOptions = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "CameraOptions" ) );
	PvGenEnum* lTLUTMode = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("TemperatureLinearMode"));
    
	// Verify all params - except name which is optional
	ASSERT( lManufacturer != NULL );
	ASSERT( lModelName != NULL );

	PvString lManufacturerStr, lModelNameStr, lInfoStr = "N/A";
	lManufacturer->GetValue( lManufacturerStr );
	lModelName->GetValue( lModelNameStr );
	lManufacturerInfo->GetValue( lInfoStr );

	CString devInfo;
	devInfo = lInfoStr.GetUnicode();
	TRACE( "Info: %S\r\n", devInfo);
   	OutputDebugString(devInfo);

	if (devInfo.Find(L"A320G") >= 0) {
		m_CamType = CAM_TYPE_A3XX;
		lWidth->SetValue(320);
		lHeight->SetValue(246);
		m_Width = 320;
		m_Height = 246;
		lPixelFormat->SetValue(0x01100007);
        m_bTLUTCapable = true;
	}
	else if (devInfo.Find(L"A645") >= 0) {
		m_CamType = CAM_TYPE_A6XX;
		lWidth->SetValue(640);
		lHeight->SetValue(483);
		m_Width = 640;
		m_Height = 483;
        lPixelFormat->SetValue(0x01100007);
        m_bTLUTCapable = true;
    	PvGenEnum* lWindowing = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("IRWindowing"));
        if (lWindowing) {
    		int64_t mode = 0;
            lWindowing->GetValue(mode);
            m_WindowingMode = (int)mode;
            if (m_WindowingMode == 1) {
                lHeight->SetValue(240);
        		m_Height = 240;
            }
            else if (m_WindowingMode == 2) {
                lHeight->SetValue(120);
        		m_Height = 120;
            }
        }
	}
	else if (devInfo.Find(L"ATAU") >= 0) {
		m_CamType = CAM_TYPE_ATAU;
        lPixelFormat->SetValue(0x01100025); // Mono14
		int64_t w = 0;
		int64_t h = 0;
		lWidth->GetValue( w );
		lHeight->GetValue( h );
		m_Width = (int)w;
		m_Height = (int)h;
    	PvGenEnum* lDigitalOutput = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("DigitalOutput"));
        lDigitalOutput->SetValue(3); // Enable
    	PvGenEnum* lCMOSBitDepth = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("CMOSBitDepth"));
        if (lCMOSBitDepth) 
            lCMOSBitDepth->SetValue(0); // 14 bit

        {
        	PvGenInteger *lR = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "R" ) );
        	PvGenFloat *lB = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "B" ) );
        	PvGenFloat *lF = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "F" ) );
        	PvGenFloat *lO = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "O" ) );
            if (lR && lB && lF && lO)
            {
    		    int64_t tmpR = 0;
                lResult = lR->GetValue(tmpR);
                m_R = (int)tmpR;
                lB->GetValue(m_B);
                lF->GetValue(m_F);
                lO->GetValue(m_O);
                m_bMeasCapable = true;
            }
        }

        if (lTLUTMode) {
            m_bTLUTCapable = true;
            lTLUTMode->SetValue(0); // Turn it off
        }

        if (lOptions) {
   		    int64_t opt = 0;
            lResult = lOptions->GetValue(opt);
            m_bSingleRange = opt & 0x2 ? false : true;
        }

	}
	else if (devInfo.Find(L"T1KX") >= 0) {
		m_CamType = CAM_TYPE_T1K;
		lWidth->SetValue(1024);
		lHeight->SetValue(770);
		m_Width = 1024;
		m_Height = 770;
        lPixelFormat->SetValue(0x01100007);
        m_bTLUTCapable = false;
    }

    if (m_CamType != CAM_TYPE_ATAU)
    {
        // Get current object parameters
        PvGenFloat *lEmissivity = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ObjectEmissivity" ) );
        PvGenFloat *lExtOptTransm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ExtOpticsTransmission" ) );
        PvGenFloat *lRelHum = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "RelativeHumidity" ) );
        PvGenFloat *lDist = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ObjectDistance" ) );
        PvGenFloat *lAmb = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ReflectedTemperature" ) );
        PvGenFloat *lAtm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "AtmosphericTemperature" ) );
        PvGenFloat *lEOT = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ExtOpticsTemperature" ) );
        
        if (lEmissivity && lExtOptTransm && lRelHum && lDist && lAmb && lAtm && lEOT)
        {
            lEmissivity->GetValue(m_Emissivity);
            lExtOptTransm->GetValue(m_ExtOptTransm);
            lRelHum->GetValue(m_RelHum);
            lDist->GetValue(m_ObjectDistance);
            lAmb->GetValue(m_AmbTemp);
            lAtm->GetValue(m_AtmTemp);
            lEOT->GetValue(m_ExtOptTemp);
        }

        // Get current measurement parameters
        PvGenFloat *lR = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "R" ) );
        PvGenFloat *lB = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "B" ) );
        PvGenFloat *lF = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "F" ) );
        if (lR && lB && lF) {
   		    double tmpR = 0;
            PvResult lr;
            lr = lR->GetValue(tmpR);
            m_R = (int)tmpR;
            lB->GetValue(m_B);
            lF->GetValue(m_F);
            if (lr.IsOK())
                m_bMeasCapable = true;
        }

        // Get gain (J1) and offset (J0)
        PvGenInteger *lJ0 = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "J0" ) );
        PvGenFloat *lJ1 = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "J1" ) );
        if (lJ0 && lJ1) {
            int64_t tmp;
            PvResult lr;
            lr = lJ1->GetValue(m_J1);
            lJ0->GetValue(tmp);
            m_J0 = (ULONG)tmp;
            if (lr.IsOK())
                m_bMeasCapable = true;
        }

        // Get spectral response
        PvGenFloat *lX = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "X" ) );
        PvGenFloat *la1 = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "alpha1" ) );
        PvGenFloat *la2 = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "alpha2" ) );
        PvGenFloat *lb1 = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "beta1" ) );
        PvGenFloat *lb2 = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "beta2" ) );
        if (lX && la1 && la2 && lb1 && lb2) {
            PvResult lr;
            lr = lX->GetValue(m_X);
            la1->GetValue(m_alpha1);
            la2->GetValue(m_alpha2);
            lb1->GetValue(m_beta1);
            lb2->GetValue(m_beta2);
            if (lr.IsOK())
                m_bMeasCapable = true;
        }

        if (m_bMeasCapable)
            doUpdateCalcConst();
    }
    else 
    {
        // Get current object parameters
        PvGenFloat *lEmissivity = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ObjectEmissivity" ) );
        PvGenFloat *lAmb = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ReflectedTemperature" ) );
        PvGenFloat *lAtm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "AtmosphericTemperature" ) );
        PvGenFloat *lEOT = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "WindowTemperature" ) );
        PvGenFloat *lExtOptTransm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "WindowTransmission" ) );

        if (lEmissivity && lAmb && lAtm && lEOT && lExtOptTransm)
        {
            lEmissivity->GetValue(m_Emissivity);
            lAmb->GetValue(m_AmbTemp);
            lAtm->GetValue(m_AtmTemp);
            lEOT->GetValue(m_ExtOptTemp);
            lExtOptTransm->GetValue(m_ExtOptTransm);
        }

        // Initiate spectral response
        m_X = 1.9;
        m_alpha1 = 0.006569;
        m_beta1 = -0.002276;
        m_alpha2 = 0.01262;
        m_beta2 = -0.00667;

        if (m_bMeasCapable)
            doUpdateCalcConst();
    }

    int64_t i64PixelFormat;
	lPixelFormat->GetValue(i64PixelFormat);
	mpvPixelFormat = (int)i64PixelFormat;

    if (m_CamType != CAM_TYPE_T1K)
    {
    	// IP
	    CString lIPStr;
    	int64_t lIPAddress = 0;
	    lIPAddressParam->GetValue( lIPAddress );
	    unsigned char *lIPPtr = reinterpret_cast<unsigned char *>( &lIPAddress );
	    lIPStr.Format( _T( "%i.%i.%i.%i" ), 
		    lIPPtr[3], lIPPtr[2], lIPPtr[1], lIPPtr[0] );

    	// MAC address
	    CString lMACStr;
    	int64_t lMACAddress;
	    lMACAddressParam->GetValue( lMACAddress );
    	unsigned char *lMACPtr = reinterpret_cast<unsigned char *>( &lMACAddress );
	    lMACStr.Format( _T( "%02X:%02X:%02X:%02X:%02X:%02X" ), 
		    lMACPtr[5], lMACPtr[4], lMACPtr[3], lMACPtr[2], lMACPtr[1], lMACPtr[0] );

        mIPEdit.SetWindowText( lIPStr );
	    mMACEdit.SetWindowText( lMACStr );
    }

	// Device name (User ID)
	CString lNameStr( "N/A" );
	if ( lNameParam != NULL )
	{
		PvString lStr;
		lNameParam->GetValue( lStr );
		lNameStr = lStr.GetUnicode();
	}

	mManufacturerEdit.SetWindowText( lManufacturerStr );
	mModelEdit.SetWindowText( lModelNameStr.GetUnicode() );
	mNameEdit.SetWindowText( lNameStr );

    mAcquiringImages = false;

	m_ctrlFrameRate.ResetContent();
	if (m_CamType == CAM_TYPE_A6XX)
	{
		int ix;
    	m_bFrameRateControlEnabled = true;
		ix = m_ctrlFrameRate.AddString(_T("50.0  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 0);

		ix = m_ctrlFrameRate.AddString(_T("25.0  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 1);

		ix = m_ctrlFrameRate.AddString(_T("12.5  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 2);

		ix = m_ctrlFrameRate.AddString(_T("6.25  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 4);

		ix = m_ctrlFrameRate.AddString(_T("3.125 Hz"));
		m_ctrlFrameRate.SetItemData(ix, 5);
	}
	else if (m_CamType == CAM_TYPE_A3XX)
	{
		int ix;
    	m_bFrameRateControlEnabled = true;
		ix = m_ctrlFrameRate.AddString(_T("60.0  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 0);
		ix = m_ctrlFrameRate.AddString(_T("30.0  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 1);
		ix = m_ctrlFrameRate.AddString(_T("15.0  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 2);
		ix = m_ctrlFrameRate.AddString(_T("9.0   Hz"));
		m_ctrlFrameRate.SetItemData(ix, 3);
		ix = m_ctrlFrameRate.AddString(_T("7.5   Hz"));
		m_ctrlFrameRate.SetItemData(ix, 4);
		ix = m_ctrlFrameRate.AddString(_T("3.75  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 5);
	}
	else if (m_CamType == CAM_TYPE_T1K)
	{
		int ix;
    	m_bFrameRateControlEnabled = true;
		ix = m_ctrlFrameRate.AddString(_T("30.0  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 1);
		ix = m_ctrlFrameRate.AddString(_T("120.0  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 6);
		ix = m_ctrlFrameRate.AddString(_T("240.0  Hz"));
		m_ctrlFrameRate.SetItemData(ix, 7);
	}

    m_ctrlPresentation.SetCurSel(0);

	if (m_CamType != CAM_TYPE_ATAU)
	{
		int64_t lValue = 0;
   		lFrameRate->GetValue( lValue );
        if (m_CamType == CAM_TYPE_A6XX && lValue > 2) {
            lValue = lValue - 1;
        }
    	else if (m_CamType == CAM_TYPE_T1K) {
            if (lValue == 1)
                lValue = 0; // 30 Hz
            else if (lValue == 6)
                lValue = 1; // 120 Hz
            else
                lValue = 2; // 240 Hz
        }
        
	    m_ctrlFrameRate.SetCurSel((int)lValue);

		lValue = 0;
  		lIRFormat->GetValue( lValue );
    	m_ctrlIRFormat.SetCurSel( (int)lValue );
	    m_ctrlUnit.ShowWindow((int)lValue > 0 ? SW_SHOW : SW_HIDE);
	    m_IRFormat = (int)lValue;
	    EnumCases();
        m_ctrlPresentation.SetCurSel(m_IRFormat == IRFMT_SIGNAL ? 0 : 1);
        m_iPresentation = (m_IRFormat == IRFMT_SIGNAL ? PRES_SIGNAL : PRES_CELSIUS);
        m_ctrlPresentation.EnableWindow( (m_IRFormat > IRFMT_SIGNAL) || m_bMeasCapable );
	}
	else {
		m_ctrlIRFormat.ResetContent();
		m_ctrlIRFormat.AddString(_T("Signal"));
        if (m_bTLUTCapable) {
    		m_ctrlIRFormat.AddString(_T("Temp. linear"));
        }

        m_ctrlPresentation.ResetContent();
        m_ctrlPresentation.AddString(_T("Signal"));
        if (m_bMeasCapable) {
            m_ctrlPresentation.AddString(_T("Celsius"));
            m_ctrlPresentation.AddString(_T("Fahrenheit"));
            m_ctrlPresentation.AddString(_T("Kelvin"));
        }
        m_ctrlPresentation.SetCurSel(0);

		m_ctrlIRFormat.SetCurSel(0);
		m_IRFormat = IRFMT_SIGNAL;

        m_ctrlRange.ResetContent();
		int ix = m_ctrlRange.AddString(_T("High Gain"));
		m_ctrlRange.SetItemData(ix, 0);
        if (!m_bSingleRange)
        {
    		ix = m_ctrlRange.AddString(_T("Low Gain"));
	    	m_ctrlRange.SetItemData(ix, 1);
        }

        if (m_bSingleRange)
        {
            m_ctrlRange.SetCurSel(0);
        }
        else
        {
        	PvGenEnum* lRange = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("SensorGainMode"));
            if (lRange) {
    	    	int64_t lValue = 0;
                lRange->GetValue(lValue);
                if (lValue == 1) {
                    // Low gain
   	    	        m_ctrlRange.SetCurSel(1);
                    m_WT = 303.0;
                    m_W4WT = 150.0;
                }
                else {
                    // High gain
   		            m_ctrlRange.SetCurSel(0);
                    m_WT = 573.0;
                    m_W4WT = 1542.0;
                }
            }
        }

        PvGenEnum* lSpeed = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("SensorFrameRate"));
        if (lSpeed) {
    		int64_t lValue = 0;
            lSpeed->GetValue(lValue);
            m_bFast = (lValue == 4); // Used for ATAU

            if (m_bFast) {
                int ix;
            	m_bFrameRateControlEnabled = true;
                ix = m_ctrlFrameRate.AddString(_T("60.0  Hz"));
        		m_ctrlFrameRate.SetItemData(ix, 4);
		        ix = m_ctrlFrameRate.AddString(_T("30.0  Hz"));
		        m_ctrlFrameRate.SetItemData(ix, 0);
                PvGenEnum* lRate = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("SensorVideoStandard"));
                lValue = 0;
                if (lRate) lRate->GetValue(lValue);
                m_ctrlFrameRate.SetCurSel(lValue == 4 ? 0 : 1);
            }
        }
	}

	if (m_CamType == CAM_TYPE_ATAU)
	{
        // This code is intended to catch changes on the FLIR AX5 GPI+ port
        // The General Purpose Input Port is connected PLC_I0
	    // Route PLC_I0 to Q7  for PLC_Interrupt_FIFO_Q7
	    mDevice->GetParameters()->SetEnumValue ("PLC_Q7_Variable0", "PLC_I0" );
    	mDevice->GetParameters()->SetEnumValue ("PLC_Q7_Operator0", "Or"     );
	    mDevice->GetParameters()->SetEnumValue ("PLC_Q7_Variable1", "Zero"   );
	    mDevice->GetParameters()->SetEnumValue ("PLC_Q7_Operator1", "Or"     );
	    mDevice->GetParameters()->SetEnumValue ("PLC_Q7_Variable2", "Zero"   );
	    mDevice->GetParameters()->SetEnumValue ("PLC_Q7_Operator2", "Or"     );
	    mDevice->GetParameters()->SetEnumValue ("PLC_Q7_Variable3", "Zero"   );

    	PvGenInteger *lBounce = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "LineDebounceFactor" ) );
        if (lBounce) {
            // Line debounce value 0 (default) disables the input debouncing
            int64_t bounceFactor = 65535; // Use max debounce, 65535 x 480 ns => 31 ms
            lBounce->SetValue(bounceFactor);
        }

	    // Enable PLC_Interrupt_FIFO_Q7
    	// Select the EventSelector as Q7
	    mDevice->GetParameters()->SetEnumValue( "EventSelector", "PLC_Interrupt_FIFO0_Q7" );
		// Enable Q7 Interrupt
  	    mDevice->GetParameters()->SetEnumValue( "EventNotification", "GigEVisionEvent");

        PvGenParameter* lGenParameter; 
		lGenParameter = mDevice->GetParameters()->Get( "PLC_Interrupt_FIFO0_IRQ_mask" );
		lGenParameter->RegisterEventSink(this);
    }
	else if (m_CamType == CAM_TYPE_T1K) {
    	// Select the EventSelector
	    mDevice->GetParameters()->SetEnumValue( "EventSelector", "PlcEventQueue0" );
	    mDevice->GetParameters()->SetEnumValue( "EventNotification", "On" );
    	PvGenInteger *lBounce = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "LineDebounceFactor" ) );
        if (lBounce) {
            // Line debounce value 0 (default) disables the input debouncing
            int64_t bounceFactor = 65535; // Use max debounce, 65535 x 480 ns => 31 ms
    	    mDevice->GetParameters()->SetEnumValue( "LineSelector", "GpioIn1" );
            lBounce->SetValue(bounceFactor);
        }
        mDevice->GetParameters()->SetStringValue("PlcEquation2", "event0=GpioIn1");
	    mDevice->GetParameters()->ExecuteCommand("PlcEquationApply");
    }

	if (m_Timer == 0)
		m_Timer = (UINT)SetTimer(0x40, 500, NULL);

    mAcquisitionStateManager = new PvAcquisitionStateManager(mDevice, mStream);
    mAcquisitionStateManager->RegisterEventSink( this );

	// Ready image reception
	StartStreaming();

	// Sync up UI
	EnableInterface();
}

// =============================================================================
void PvSimpleUISampleDlg::Disconnect()
{
   	if ( mDevice != NULL )
	{
        // Unregister all events of the parameters in the device's node map
        PvGenParameterArray *lGenDevice = mDevice->GetParameters();
        for ( uint32_t i = 0; i < lGenDevice->GetCount(); i++ )
        {
	        lGenDevice->Get( i )->UnregisterEventSink( this );
        }
    }

    // Close all configuration child windows
	CloseGenWindow( &mDeviceWnd );
	CloseGenWindow( &mCommunicationWnd );
	CloseGenWindow( &mStreamParametersWnd );

	// If streaming, stop streaming
	StopStreaming();

    // Release acquisition state manager
    if (mAcquisitionStateManager != NULL) {
        delete mAcquisitionStateManager;
        mAcquisitionStateManager = NULL;
    }

	// Reset device ID - can be called by the destructor when the window
	// no longer exists, be careful...
	if ( GetSafeHwnd() != 0 )
	{
		mManufacturerEdit.SetWindowText( _T( "" ) );
		mModelEdit.SetWindowText( _T( "" ) );
		mIPEdit.SetWindowText( _T( "" ) );
		mMACEdit.SetWindowText( _T( "" ) );
		mNameEdit.SetWindowText( _T( "" ) );
	}

	if (m_Timer != 0) {
		KillTimer(m_Timer);
		m_Timer = 0;
	}

    if ( mDevice != NULL )
	{
        PvDevice::Free( mDevice );
        mDevice = NULL;
	}
    
	if ( mStream != NULL )
	{
        PvStream::Free( mStream );
        mStream = NULL;
	}
	
	EnableInterface();
}


// =============================================================================
void PvSimpleUISampleDlg::OnBnClickedDeviceButton()
{
	if ( !mDevice->IsConnected() )
	{
		return;
	}

	ShowGenWindow( 
		&mDeviceWnd, 
		mDevice->GetParameters(), 
		_T( "GEV Device Control" ) );
}


// =============================================================================
void PvSimpleUISampleDlg::OnBnClickedDeviceEvents()
{
}


// =============================================================================
void PvSimpleUISampleDlg::OnBnClickedLinkButton()
{
	if ( !mDevice->IsConnected() )
	{
		return;
	}

    ShowGenWindow( 
		&mCommunicationWnd, 
        mDevice->GetCommunicationParameters(), 
		_T( "Communication Control" ) );
}


// =============================================================================
void PvSimpleUISampleDlg::OnBnClickedStreamparamsButton()
{
	ShowGenWindow( 
		& mStreamParametersWnd, 
		mStream->GetParameters(), 
		_T( "Image Stream Control" ) );
}


// =============================================================================
void PvSimpleUISampleDlg::ShowGenWindow( PvGenBrowserWnd **aWnd, PvGenParameterArray *aParams, const CString &aTitle )
{
	if ( ( *aWnd ) != NULL )
	{
		if ( ( *aWnd )->GetHandle() != 0 )
		{
			CWnd lWnd;
			lWnd.Attach( ( *aWnd )->GetHandle() );

			// Window already visible, give it focus and bring it on top
			lWnd.BringWindowToTop();
			lWnd.SetFocus();

			lWnd.Detach();
			return;
		}

		// Window object exists but was closed/destroyed. Free it before re-creating
		CloseGenWindow( aWnd );
	}

	// Create, assigne parameters, set title and show modeless
	( *aWnd ) = new PvGenBrowserWnd;
	( *aWnd )->SetTitle( PvString( aTitle ) );
	( *aWnd )->SetGenParameterArray( aParams );
	( *aWnd )->ShowModeless( GetSafeHwnd() );
}

// =============================================================================
void PvSimpleUISampleDlg::CloseGenWindow( PvGenBrowserWnd **aWnd )
{
	// If the window object does not even exist, do nothing
	if ( ( *aWnd ) == NULL )
	{
		return;
	}

	// If the window object exists and is currently created (visible), close/destroy it
	if ( ( *aWnd )->GetHandle() != 0 )
	{
		( *aWnd )->Close();
	}

	// Finally, release the window object
	delete ( *aWnd );
	( *aWnd ) = NULL;
}

// =============================================================================
void PvSimpleUISampleDlg::StartStreaming()
{
	// Create display thread
	mThreadDisplay = new Thread( this, &mDisplay, mDevice, mPipeline, m_Width, m_Height, m_CamType, m_WindowingMode );

	TCHAR szPath[_MAX_PATH];
	SHGetSpecialFolderPath(NULL, szPath, CSIDL_MYPICTURES, FALSE);
	mThreadDisplay->m_SnapshotPath = CString(szPath);

	// Start threads
	mThreadDisplay->Start();
	mThreadDisplay->SetPriority( THREAD_PRIORITY_ABOVE_NORMAL );

    // Retrieve Pixel Size Byte per pixel (Bpp) information
 	miPixelSize =((mpvPixelFormat & 0x00FF0000) >> 16)/8;
 
	mPipeline->SetBufferSize( m_Width * m_Height * miPixelSize ); 
	mPipeline->SetBufferCount( 8 );

    // Start pipeline
    mPipeline->Start();
}

// =============================================================================
void PvSimpleUISampleDlg::StopStreaming()
{
    // Stop display thread
	if ( mThreadDisplay != NULL )
	{
		mThreadDisplay->Stop();
		delete mThreadDisplay;
		mThreadDisplay = NULL;
	}

	// Stop stream thread
	if ( mPipeline != NULL )
	{
        if ( mPipeline->IsStarted() )
        {
		    mPipeline->Stop();
        }
    }
}

// =============================================================================
void PvSimpleUISampleDlg::StartAcquisition()
{
	PvGenEnum *lMode = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "AcquisitionMode" ) );
	PvGenInteger *lTLParamsLocked = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "TLParamsLocked" ) );

    // Never hurts to start fresh...
    mPipeline->Reset();

    // Reset stream statistics
    mStream->GetParameters()->ExecuteCommand( "Reset" );

	PvResult lResult = PvResult::Code::NOT_INITIALIZED;

	PvString lStr;
	lMode->GetValue( lStr );
	CString lModeStr = lStr.GetUnicode();
	if ( lModeStr.Find( _T( "Continuous" ) ) >= 0 )
	{
		// We are streaming, lock the TL parameters
		if ( lTLParamsLocked != NULL ) 
		{
			lResult = lTLParamsLocked->SetValue( 1 );
		}

        mAcquiringImages = true;
	}
	else if ( ( lModeStr.Find( _T( "Multi" ) ) >= 0 ) || 
			  ( lModeStr.Find( _T( "Single" ) ) >= 0 ) )
	{
		// We are streaming, lock the TL parameters
		if ( lTLParamsLocked != NULL ) 
		{
			mbWorkaround = true;
			lTLParamsLocked->SetValue( 1 );
		}

		// We are done streaming, unlock the TL parameters
		if ( lTLParamsLocked != NULL ) 
		{
			lTLParamsLocked->SetValue( 0 );
			Sleep(100);
            mbWorkaround = false;
		}
	}

    mAcquisitionStateManager->Start();

	m_ctrlAreaSetup.EnableWindow(TRUE);
	m_ctrlPalette.EnableWindow(TRUE);
	m_btnSaveImage.EnableWindow(m_IRFormat == IRFMT_SIGNAL);
	m_btnSaveSEQ.EnableWindow(m_IRFormat == IRFMT_SIGNAL);
	m_btnSaveBMP.EnableWindow(TRUE);
	m_btnSaveRAW.EnableWindow(TRUE);

    if ( mThreadDisplay) {
        mThreadDisplay->resetStats();
	}
}
// =============================================================================
void PvSimpleUISampleDlg::StopAcquisition()
{
    mAcquiringImages = false;
    mAcquisitionStateManager->Stop();

    // TLParamsLocked - Optional
	PvGenInteger *lTLParamsLocked = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "TLParamsLocked" ) );
	if ( lTLParamsLocked != NULL )
	{
		lTLParamsLocked->SetValue( 0 );
	}

    mStream->AbortQueuedBuffers();
	Sleep(100);
    mbWorkaround = false;
	m_ctrlAreaSetup.EnableWindow(FALSE);
	m_ctrlPalette.EnableWindow(FALSE);
	m_btnSaveImage.EnableWindow(FALSE);
	m_btnSaveBMP.EnableWindow(FALSE);
	m_btnSaveRAW.EnableWindow(FALSE);
	m_btnSaveSEQ.EnableWindow(FALSE);
}

// =============================================================================
void PvSimpleUISampleDlg::OnBnClickedStart()
{
    if ( !mDevice->IsConnected() )
    {
        return;
    }

    StartAcquisition();
	EnableInterface();
}

// =============================================================================
void PvSimpleUISampleDlg::OnBnClickedStop()
{
    if ( !mDevice->IsConnected() )
    {
        return;
    }

    StopAcquisition();
	EnableInterface();
}

// =============================================================================
void PvSimpleUISampleDlg::OnParameterUpdate( PvGenParameter *aParameter )
{
	bool bBufferResize =false;
	PvString lName;
    static int trig1 = 0;
    static int trig2 = 0;

	if ( !aParameter->GetName( lName ).IsOK() )
	{
		ASSERT( 0 ); // Totally unexpected	
		return;
	}

    /*
    // If the AcquisitionStop node is updated, make sure to reflect its new state on the UI
	if ( ( lName == "AcquisitionStop" ) &&
		 ( mStopButton.GetSafeHwnd() != 0 ) )
	{
		bool lAvailable = false;
		if ( aParameter->IsAvailable( lAvailable ).IsOK() )
		{
			mStopButton.EnableWindow( lAvailable );
		}
	}*/

	if (  lName == "PLC_Interrupt_FIFO0_IRQ_mask" )
	{	
        //TRACE(_T("Trig\r\n"));
        CString s;
        trig1++;
        s.Format(_T("%d"), trig1);
        m_ctrlTrigCount1.SetWindowText(s);
	}

    if (  lName == "PlcEventQueueLutInputs" )
    {
        PvGenInteger* lPvGenInteger;
		int64_t value = 0;
		lPvGenInteger = dynamic_cast<PvGenInteger *>(mDevice->GetParameters()->Get( "PlcEventQueueLutInputs" ) );
        if (lPvGenInteger)
		    lPvGenInteger->GetValue(value);
        TRACE(_T("Trig %d\r\n"), (int)value);
        CString s;
        if (value & 0x04) {
            trig1++;
            s.Format(_T("%d"), trig1);
            m_ctrlTrigCount1.SetWindowText(s);
        }
        else if (value & 0x08) {
            trig2++;
            s.Format(_T("%d"), trig2);
            m_ctrlTrigCount2.SetWindowText(s);
        }
    }

	if (  lName == "Width"  )
	{	
		if( !mbWorkaround  )
		{
			PvGenInteger* lPvGenInteger;
			int64_t value;
			lPvGenInteger = dynamic_cast<PvGenInteger *>(mDevice->GetParameters()->Get( "Width" ) );
			lPvGenInteger->GetValue(value);
			if(m_Width != (int)value)
			{
				m_Width = (int)value;
				bBufferResize  =true;
			}
		}
	}
	
	if (  lName == "Height" )
	{	
		if( !mbWorkaround  )
		{
			PvGenInteger* lPvGenInteger;
			int64_t value;
			lPvGenInteger = dynamic_cast<PvGenInteger *>(mDevice->GetParameters()->Get( "Height" ) );
			lPvGenInteger->GetValue(value);
			if( m_Height != (int)value)
			{	
				m_Height = (int)value;
				bBufferResize  =true;
			}
		}
	}
	
	if (  lName == "PixelFormat" )
	{
		if( !mbWorkaround  )
		{
			int64_t i64PixelFormat;
			PvGenEnum* lPvGenEnum;
			lPvGenEnum = dynamic_cast<PvGenEnum *>(mDevice->GetParameters()->Get("PixelFormat"));
			lPvGenEnum->GetValue(i64PixelFormat);
			if(mpvPixelFormat != (int)i64PixelFormat)
			{
				mpvPixelFormat = (int)i64PixelFormat;
				miPixelSize =1;
 				miPixelSize =((mpvPixelFormat & 0x00FF0000) >> 16)/8;
				bBufferResize  =true;
			}
		}
	}

    if (  lName == "ObjectEmissivity" ) {
        PvGenFloat *lEmissivity = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "ObjectEmissivity" ) );
        lEmissivity->GetValue(m_Emissivity);
        doUpdateCalcConst();
    }
    else if (  lName == "ExtOpticsTransmission" ) {
        PvGenFloat *lExtOptTransm = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "ExtOpticsTransmission" ) );
        lExtOptTransm->GetValue(m_ExtOptTransm);
        doUpdateCalcConst();
    }
    else if (  lName == "ExtOpticsTemperature" ) {
        PvGenFloat *lEOT = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "ExtOpticsTemperature" ) );
        lEOT->GetValue(m_ExtOptTemp);
        doUpdateCalcConst();
    }
    else if (  lName == "RelativeHumidity" ) {
        PvGenFloat *lRelHum = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "RelativeHumidity" ) );
        lRelHum->GetValue(m_RelHum);
        doUpdateCalcConst();
    }
    else if (  lName == "ObjectDistance" ) {
        PvGenFloat *lDist = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "ObjectDistance" ) );
        lDist->GetValue(m_ObjectDistance);
        doUpdateCalcConst();
    }
    else if (  lName == "ReflectedTemperature" ) {
        PvGenFloat *lAmb = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "ReflectedTemperature" ) );
        lAmb->GetValue(m_AmbTemp);
        doUpdateCalcConst();
    }
    else if (  lName == "AtmosphericTemperature" ) {
        PvGenFloat *lAtm = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "AtmosphericTemperature" ) );
        lAtm->GetValue(m_AtmTemp);
        doUpdateCalcConst();
    }
    else if (  lName == "WindowTransmission" ) {
        PvGenFloat *lExtOptTransm = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "WindowTransmission" ) );
        lExtOptTransm->GetValue(m_ExtOptTransm);
        doUpdateCalcConst();
    }
    else if (  lName == "WindowTemperature" ) {
        PvGenFloat *lEOT = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "WindowTemperature" ) );
        lEOT->GetValue(m_ExtOptTemp);
        doUpdateCalcConst();
    }

  	if(bBufferResize)
	{
		bool bPipelineRestart = false;
		if( mPipeline->IsStarted())
		{	
			mPipeline->Stop();
			bPipelineRestart = true;
		}
		mPipeline->SetBufferSize( m_Width * m_Height * miPixelSize ); 
		if(bPipelineRestart)
			mPipeline->Start();
	}
}

void PvSimpleUISampleDlg::EnumCases()
{
    PvResult lr;
	PvGenInteger *lNumCases = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "NumCases" ) );
	PvGenInteger *lQueryCase = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "QueryCase" ) );
	PvGenFloat *lLoLimit = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "QueryCaseLowLimit" ) );
	PvGenFloat *lHiLimit = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "QueryCaseHighLimit" ) );
	PvGenBoolean *lEnabled = dynamic_cast<PvGenBoolean *>( mDevice->GetParameters()->Get( "QueryCaseEnabled" ) );
	PvGenInteger *lCase = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "CurrentCase" ) );

	int64_t Value = 0;
	lr = lNumCases->GetValue( Value );
    if (lr.IsFailure())
        return;

	double lo, hi;
	long i;
	bool enabled;
	int numCases = (int)Value;
    DWORD itemData;

	m_ctrlRange.ResetContent();
	for (i = 0; i < numCases; i++) 
	{
		// Set case selector
		lQueryCase->SetValue( i ); 

		lLoLimit->GetValue( lo );
		lHiLimit->GetValue( hi );

		lEnabled->GetValue( enabled );

		if (enabled) {
			CString s;
			s.Format(_T("%.0f - %.0f C"), (lo - 273.15f), (hi - 273.15f));
			int ix = m_ctrlRange.AddString(s);
			m_ctrlRange.SetItemData(ix, i);
		}

	}

	lCase->GetValue( Value );
    for (i = 0; i < m_ctrlRange.GetCount(); i++)
    {
        itemData = (DWORD)m_ctrlRange.GetItemData(i);
        if (itemData == (DWORD)Value) {
    	    m_ctrlRange.SetCurSel( i );
            break;
        }
    }
}

// =============================================================================
void PvSimpleUISampleDlg::OnMove(int x, int y)
{
	CDialog::OnMove(x, y);
    //mDisplay.UpdateBounds();
}


void PvSimpleUISampleDlg::OnCbnSelchangePalette()
{
	int sel = m_ctrlPalette.GetCurSel();
	if ( mThreadDisplay != NULL )
	{
		mThreadDisplay->SetPalette( sel );
	}
}

void PvSimpleUISampleDlg::OnBnClickedAutofocus()
{
	PvGenCommand *lAF = dynamic_cast<PvGenCommand *>( mDevice->GetParameters()->Get( "AutoFocus" ) );
	if (lAF) {
		lAF->Execute();
	}
}

void PvSimpleUISampleDlg::OnBnClickedSaveImage()
{
	if ( mThreadDisplay != NULL ) {
		mThreadDisplay->SaveImage(FMT_FFF);
	}
}

void PvSimpleUISampleDlg::OnBnClickedSaveBmp()
{
	if ( mThreadDisplay != NULL ) {
		mThreadDisplay->SaveImage(FMT_BMP);
	}
}

void PvSimpleUISampleDlg::OnBnClickedSaveRaw()
{
	if ( mThreadDisplay != NULL ) {
		mThreadDisplay->SaveImage(FMT_RAW);
	}
}

void PvSimpleUISampleDlg::OnBnClickedSaveSeq()
{
	if ( mThreadDisplay != NULL ) {
        UpdateData();
        m_btnSaveSEQ.EnableWindow(FALSE);
    	m_btnSaveSEQ.SetBGColor(RED, TRUE);
        m_bRecording = true;
        mThreadDisplay->SetSequenceFrames(m_uiSeqFrames);
		mThreadDisplay->SaveImage(FMT_SEQ);
	}
}

void PvSimpleUISampleDlg::OnCbnSelchangeFrate()
{
	int sel = m_ctrlFrameRate.GetCurSel();
	int64_t v = m_ctrlFrameRate.GetItemData(sel);

	if (m_CamType == CAM_TYPE_ATAU)
    {
    	PvGenEnum *lFrameRate = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "SensorVideoStandard" ) );
	    if (lFrameRate) {
		    lFrameRate->SetValue( v );
	    }
    	if ( mThreadDisplay != NULL ) {
            mThreadDisplay->resetStats();
	    }
    }
    else
    {
    	PvGenEnum *lFrameRate = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "IRFrameRate" ) );
	    if (lFrameRate) {
		    lFrameRate->SetValue( v );
	    }
    }
}

void PvSimpleUISampleDlg::OnCbnSelchangePixelFormat()
{
	int sel = m_ctrlIRFormat.GetCurSel();

   	PvGenEnum *lIRFormat = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "IRFormat" ) );
    if (lIRFormat) {
	    lIRFormat->SetValue( sel );
    }

    if (m_CamType == CAM_TYPE_ATAU && m_bTLUTCapable)
    {
        if (sel == 0)
        {
            // Signal mode requested - turn off temperature linear mode
           	PvGenEnum *lTLMode = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "TemperatureLinearMode" ) );
            if (lTLMode) {
	            lTLMode->SetValue( 0 );
            }
            m_ctrlPresentation.EnableWindow(TRUE);
           	m_IRFormat = IRFMT_SIGNAL;
        }
        else
        {
            // Enable temperature linear mode
           	PvGenEnum *lTLMode = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "TemperatureLinearMode" ) );
            if (lTLMode) {
	            lTLMode->SetValue( 1 );
            }

            // Read current measurement range in order to select a proper resolution
       	    int range = m_ctrlRange.GetCurSel();
            int res;
            if (range == 0) 
            {
                res = 1; // High gain mode
            	m_IRFormat = IRFMT_TEMPLIN_HIRES;
            }
            else {
                res = 0; // Low gain mode
            	m_IRFormat = IRFMT_TEMPLIN_LORES;
            }

           	PvGenEnum *lTLRes = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "TemperatureLinearResolution" ) );
            if (lTLRes) {
	            lTLRes->SetValue( res );
            }

            // Force temperature presentation if current display is signal
            if (m_iPresentation == PRES_SIGNAL)
            {
                m_iPresentation = PRES_CELSIUS;
                m_ctrlPresentation.SetCurSel(1);
                OnCbnSelchangePresentation();
            }
            m_ctrlPresentation.EnableWindow(TRUE);
        }
    }
    else {
    	m_IRFormat = sel;
    }

	if ( mThreadDisplay != NULL ) {
		mThreadDisplay->SetIRFormat( m_IRFormat );
	}

    if (m_CamType != CAM_TYPE_ATAU) 
    {
        if (m_IRFormat > IRFMT_SIGNAL) 
        {
            // Force temperature presentation since the pixels represent absolute temperatures
            m_iPresentation = PRES_CELSIUS;
            m_ctrlPresentation.SetCurSel(1);
            m_ctrlPresentation.EnableWindow(TRUE);
        }
        else 
        {
            m_iPresentation = PRES_SIGNAL;
            m_ctrlPresentation.SetCurSel(0);
            m_ctrlPresentation.EnableWindow(m_bMeasCapable);
        }
    }

    if (m_iPresentation == PRES_SIGNAL)
	    m_ctrlUnit.ShowWindow(SW_HIDE);
    else if (m_iPresentation == PRES_CELSIUS)
    {
	    m_ctrlUnit.ShowWindow(SW_SHOW);
        m_ctrlUnit.SetWindowText(_T("C"));
    }
    else if (m_iPresentation == PRES_FAHRENHEIT)
    {
	    m_ctrlUnit.ShowWindow(SW_SHOW);
        m_ctrlUnit.SetWindowText(_T("F"));
    }
    else if (m_iPresentation == PRES_KELVIN)
    {
	    m_ctrlUnit.ShowWindow(SW_SHOW);
        m_ctrlUnit.SetWindowText(_T("K"));
    }

   	m_ctrlCalibrate.EnableWindow((m_CamType == CAM_TYPE_ATAU) && m_bMeasCapable && (m_IRFormat == IRFMT_SIGNAL));
	m_btnSaveImage.EnableWindow(mAcquiringImages && (m_IRFormat == IRFMT_SIGNAL));
	m_btnSaveSEQ.EnableWindow(mAcquiringImages && (m_IRFormat == IRFMT_SIGNAL));
	OnBnClickedOneshotadj();
}

void PvSimpleUISampleDlg::OnCbnSelchangePresentation()
{
	int sel = m_ctrlPresentation.GetCurSel();
    m_iPresentation = sel;

    if (m_iPresentation == PRES_SIGNAL)
	    m_ctrlUnit.ShowWindow(SW_HIDE);
    else if (m_iPresentation == PRES_CELSIUS)
    {
	    m_ctrlUnit.ShowWindow(SW_SHOW);
        m_ctrlUnit.SetWindowText(_T("C"));
    }
    else if (m_iPresentation == PRES_FAHRENHEIT)
    {
	    m_ctrlUnit.ShowWindow(SW_SHOW);
        m_ctrlUnit.SetWindowText(_T("F"));
    }
    else if (m_iPresentation == PRES_KELVIN)
    {
	    m_ctrlUnit.ShowWindow(SW_SHOW);
        m_ctrlUnit.SetWindowText(_T("K"));
    }

	if (!m_bAutoAdjust)
    {
        // Do single adjustment in order to get correct scale units
		m_bAdjustOnce = true;
	}
}

void PvSimpleUISampleDlg::OnCbnSelchangeRange()
{
	if (m_CamType == CAM_TYPE_ATAU)
    {
        int64_t lValue;
    	PvGenEnum *lRange = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "SensorGainMode" ) );
	    int sel = m_ctrlRange.GetCurSel();
        if (sel == 0) {
            lValue = 2; // High gain mode
            m_WT = 573.0;
            m_W4WT = 1542.0;
        }
        else {
            lValue = 1; // Low gain mode
            m_WT = 303.0;
            m_W4WT = 150.0;
        }
        lRange->SetValue(lValue);

        if (m_bMeasCapable) 
        {
            // Update RBF0
           	PvGenInteger *lR = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "R" ) );
       	    PvGenFloat *lB = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "B" ) );
           	PvGenFloat *lF = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "F" ) );
           	PvGenFloat *lO = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "O" ) );

            if (lR) {
                int retry = 0;
                int64_t tmpR = m_R;
                while ((m_R == (int)tmpR) && (retry < 50)) 
                {
                    Sleep(100);
                    lR->GetValue(tmpR);
                    retry++;
                }
                //TRACE("retry=%d\r\n", retry);
            }

            if (lR && lB && lF && lO)
            {
                PvResult lResult;
                int64_t tmpR = 0;
                lResult = lR->GetValue(tmpR);
                m_R = (int)tmpR;
                lB->GetValue(m_B);
                lF->GetValue(m_F);
                lO->GetValue(m_O);
                TRACE("R=%d\r\n", m_R);
                TRACE("B=%.1f\r\n", m_B);
                TRACE("F=%.1f\r\n", m_F);
                TRACE("O=%.1f\r\n", m_O);
            }
        }

        if (m_bTLUTCapable && (m_IRFormat > IRFMT_SIGNAL)) 
        {
           	PvGenEnum *lTLRes = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "TemperatureLinearResolution" ) );
            if (lTLRes) {
                int res = sel == 0 ? 1 : 0;
	            lTLRes->SetValue( res );
            }

            if (sel == 0)
                m_IRFormat = IRFMT_TEMPLIN_HIRES; // High gain and high resolution
            else
                m_IRFormat = IRFMT_TEMPLIN_LORES; // Low gain and low resolution
        }
    }
    else
    {
    	PvGenInteger *lCase = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "CurrentCase" ) );
	    int sel = m_ctrlRange.GetCurSel();
    	int64_t Value = m_ctrlRange.GetItemData(sel);
	    lCase->SetValue( Value );
    }

    m_CalcCheckNeeded = 10;
}

USHORT PvSimpleUISampleDlg::TauToSignal(float C)
{
    USHORT s;
	float K = C + 273.15f;
    s = (USHORT)(m_R /(exp(m_B/K) - m_F) + m_O);
    return s;
}

float PvSimpleUISampleDlg::TauToTemp(USHORT sig)
{
    float T;
    double objSig;
    objSig = powToObjSig((double)sig);
    T = (float)(m_B / log(m_R /(objSig - m_O) + m_F));
    return T;
}

void PvSimpleUISampleDlg::OnTimer(UINT_PTR nIDEvent)
{
	if (nIDEvent == 0x40) 
	{
        updateCalcParams();

		if (mThreadDisplay)
		{
			USHORT lo, hi, v, vc, avg, amin, amax;
			DWORD trig1, trig2;
			CString s, sc, sLo, sHi, szAvg, szMin, szMax;
            CString unit;
            CTemperature T;

            if (m_iPresentation == PRES_CELSIUS) {
                T.setUnit(CTemperature::Celsius);
                unit = _T("C");
            }
            else if (m_iPresentation == PRES_FAHRENHEIT) {
                T.setUnit(CTemperature::Fahrenheit);
                unit = _T("F");
            }
            else if (m_iPresentation == PRES_KELVIN) {
                T.setUnit(CTemperature::Kelvin);
                unit = _T("K");
            }

			avg = mThreadDisplay->getAreaAvg();
			amin = mThreadDisplay->getAreaMin();
			amax = mThreadDisplay->getAreaMax();
			v = mThreadDisplay->getSpot();
			vc = mThreadDisplay->getCenterSpot();
			lo = mThreadDisplay->getScaleLo();
			hi = mThreadDisplay->getScaleHi();
			trig1 = mThreadDisplay->getTrigCount(1);
			trig2 = mThreadDisplay->getTrigCount(2);

            //TRACE("OnTimer: IRFormat=%d pres=%d\r\n", m_IRFormat, m_iPresentation);

			if (m_IRFormat == IRFMT_SIGNAL) 
            {
                // Signal mode
    			if (m_iPresentation == PRES_SIGNAL)
                {
       		    	s.Format(_T("%d"), v);
   	    	    	sc.Format(_T("%d"), vc);
   		    		szAvg.Format(_T("%d"), avg);
    		    	szMin.Format(_T("%d"), amin);
	    		    szMax.Format(_T("%d"), amax);
    		    	s.Format(_T("%d"), v);
	    		    sLo.Format(_T("%d"), lo);
   		    		sHi.Format(_T("%d"), hi);
                }
                else
                {
        			if (m_CamType == CAM_TYPE_ATAU && m_bMeasCapable)
	        		{
                        T = TauToTemp(v);
    			    	s.Format(_T("%.1f %s"), T.Value(), unit);
                        T = TauToTemp(vc);
        				sc.Format(_T("%.1f %s"), T.Value(), unit);
                        T = TauToTemp(avg);
    		    		szAvg.Format(_T("%.1f %s"), T.Value(), unit);
                        T = TauToTemp(amin);
    				    szMin.Format(_T("%.1f %s"), T.Value(), unit);
                        T = TauToTemp(amax);
        				szMax.Format(_T("%.1f %s"), T.Value(), unit);
                        T = TauToTemp(lo);
    		    		sLo.Format(_T("%.1f"), T.Value());
                        T = TauToTemp(hi);
    				    sHi.Format(_T("%.1f"), T.Value());
	    		    }
        			else if (m_CamType != CAM_TYPE_ATAU && m_bMeasCapable)
	        		{
                        T = imgToTemp(v);
                        s.Format(_T("%.1f"), T.Value());
                        T = imgToTemp(vc);
        				sc.Format(_T("%.1f"), T.Value());
                        T = imgToTemp(avg);
    		    		szAvg.Format(_T("%.1f"), T.Value());
                        T = imgToTemp(amin);
    				    szMin.Format(_T("%.1f"), T.Value());
                        T = imgToTemp(amax);
        				szMax.Format(_T("%.1f"), T.Value());
                        T = imgToTemp(lo);
    		    		sLo.Format(_T("%.1f"), T.Value());
                        T = imgToTemp(hi);
    				    sHi.Format(_T("%.1f"), T.Value());
                    }
                }
			}
			else if (m_IRFormat == IRFMT_TEMPLIN_LORES) 
            {
                // Temperature linear mode - low resolution
       			if (m_iPresentation == PRES_SIGNAL)
                {
       				szAvg.Format(_T("%d"), avg);
    	    		szMin.Format(_T("%d"), amin);
	    		    szMax.Format(_T("%d"), amax);
    		    	s.Format(_T("%d"), v);
		    	    sc.Format(_T("%d"), vc);
                    sLo.Format(_T("%d"), lo);
        			sHi.Format(_T("%d"), hi);
                }
    			else if (m_CamType == CAM_TYPE_ATAU && m_bTLUTCapable)
                {
                    T = (float)(avg * 0.4f);
      				szAvg.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(amin * 0.4f);
       	    		szMin.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(amax * 0.4f);
	    		    szMax.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(v * 0.4f);
	   		    	s.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(vc * 0.4f);
		    	    sc.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(lo * 0.4f);
                    sLo.Format(_T("%.1f"), T.Value());
                    T = (float)(hi * 0.4f);
	       			sHi.Format(_T("%.1f"), T.Value());
                }
                else // A6xx or A3xx
                {
                    T = (float)(avg / 10.0f);
      				szAvg.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(amin / 10.0f);
    	    		szMin.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(amax / 10.0f);
	    		    szMax.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(v / 10.0f);
    		    	s.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(vc / 10.0f);
    	    	    sc.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(lo / 10.0f);
    			    sLo.Format(_T("%.1f"), T.Value());
                    T = (float)(hi / 10.0f);
	       			sHi.Format(_T("%.1f"), T.Value());
                }
			}
			else if (m_IRFormat == IRFMT_TEMPLIN_HIRES) 
            {
                // Temperature linear mode - high resolution
       			if (m_iPresentation == PRES_SIGNAL)
                {
       				szAvg.Format(_T("%d"), avg);
    	    		szMin.Format(_T("%d"), amin);
	    		    szMax.Format(_T("%d"), amax);
    		    	s.Format(_T("%d"), v);
		    	    sc.Format(_T("%d"), vc);
                    sLo.Format(_T("%d"), lo);
        			sHi.Format(_T("%d"), hi);
                }
    			else if (m_CamType == CAM_TYPE_ATAU && m_bTLUTCapable)
                {
                    T = (float)(avg * 0.04f);
    				szAvg.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(amin * 0.04f);
	    			szMin.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(amax * 0.04f);
                    szMax.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(v * 0.04f);
			    	s.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(vc * 0.04f);
				    sc.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(lo * 0.04f);
    				sLo.Format(_T("%.1f"), T.Value());
                    T = (float)(hi * 0.04f);
	    			sHi.Format(_T("%.1f"), T.Value());
                }
                else
                {
                    T = (float)(avg / 100.0f);
    				szAvg.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(amin / 100.0f);
	    			szMin.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(amax / 100.0f);
		    		szMax.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(v / 100.0f);
			    	s.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(vc / 100.0f);
				    sc.Format(_T("%.1f %s"), T.Value(), unit);
                    T = (float)(lo / 100.0f);
                    sLo.Format(_T("%.1f"), T.Value());
                    T = (float)(hi / 100.0f);
	    			sHi.Format(_T("%.1f"), T.Value());
                }
			}

            if (!m_bFlyingSpotActive) s.Empty();
		    m_ctrlSpot.SetWindowText(s);
		    m_ctrlCenterSpot.SetWindowText(sc);

			if (m_bAutoAdjust || m_bAdjustOnce)
			{
				m_ctrlScaleLow.SetWindowText(sLo);
				m_ctrlScaleHigh.SetWindowText(sHi);
				m_bAdjustOnce = false;
			}

            if (m_bAreaActive) {
                m_ctrlAreaAvg.SetWindowText(szAvg);
	    		m_ctrlAreaMin.SetWindowText(szMin);
		    	m_ctrlAreaMax.SetWindowText(szMax);
            }

			if (m_CamType == CAM_TYPE_A3XX || m_CamType == CAM_TYPE_A6XX)
			{
				s.Format(_T("%d"), trig1);
				m_ctrlTrigCount1.SetWindowText(s);
				s.Format(_T("%d"), trig2);
				m_ctrlTrigCount2.SetWindowText(s);
                {
                    int64_t state0 = 0, state1 = 0;
                    EnterCriticalSection( &m_csPort );
	    			PvGenInteger *lDigPort = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "CurrentPort" ) );
		    		PvGenEnum *lDigState = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "CurrentIOState" ) );
                    if (lDigPort && lDigState) {
                        lDigPort->SetValue( 0 );
    				    lDigState->GetValue(state0);
                        lDigPort->SetValue( 1 );
    				    lDigState->GetValue(state1);
                    }
                    m_ctrlPort1State.SetWindowText(state0 == 0 ? _T("Low") : _T("High"));
                    m_ctrlPort2State.SetWindowText(state1 == 0 ? _T("Low") : _T("High"));
                    LeaveCriticalSection( &m_csPort );
                }
			}

			m_iLogCount++;
			if (m_CamType == CAM_TYPE_ATAU && m_bLogging && m_iLogCount >= m_iLogInterval)
			{
				PvGenFloat *fTemp = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "SensorTemperature" ) );
				PvGenFloat *fTempHouse = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "HousingTemperature" ) );
				//PvGenFloat *fSpot = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "Spot" ) );
				double Sensor1, Sensor2, SpotValue;
				Sensor1 = 0;
                Sensor2 = 0;
                SpotValue = 0;

                if (fTemp) {
    				fTemp->GetValue( Sensor1 );
                }
                if (fTempHouse) {
    				fTempHouse->GetValue( Sensor2 );
                }

       			vc = mThreadDisplay->getCenterSpot();
    			if (m_IRFormat == IRFMT_SIGNAL)
                {
                    SpotValue = TauToTemp(vc) - 273.15f;
                }
                else if (m_IRFormat == IRFMT_TEMPLIN_LORES) 
                {
    			    if (m_CamType == CAM_TYPE_ATAU && m_bTLUTCapable)
                    {
				        SpotValue = (double)(vc * 0.1f) - 273.15f;
                    }
                }
                else if (m_IRFormat == IRFMT_TEMPLIN_HIRES) 
                {
    			    if (m_CamType == CAM_TYPE_ATAU && m_bTLUTCapable)
                    {
				        SpotValue = (double)(vc * 0.01f) - 273.15f;
                    }
                }

                mThreadDisplay->Log(Sensor1, Sensor2, SpotValue);
				m_iLogCount = 0;
			}

            double fps = mThreadDisplay->getFPS();
            DWORD frc = mThreadDisplay->getFrameCount();

            if (mAcquiringImages)
                s.Format(_T("Stream: %d images %.1f FPS"), frc, fps);
            else
                s.Format(_T("Stream: %d images N/A FPS"), frc, fps);
            m_ctrlStatus.SetWindowText(s);

            if (m_bRecording) {
                if (!mThreadDisplay->IsRecording()) {
                    m_btnSaveSEQ.EnableWindow(TRUE);
                	m_btnSaveSEQ.SetBGColor(WHITE, TRUE);
                    m_bRecording = false;
                }
            }
		}
	}

	__super::OnTimer(nIDEvent);
}

void PvSimpleUISampleDlg::OnBnClickedAutoadj()
{
	int state = m_btnAutoAdjust.GetCheck();
	m_bAutoAdjust = state == BST_CHECKED ? true : false;
	m_btnAdjustOnce.EnableWindow(state != BST_CHECKED);
	m_ctrlScaleLow.EnableWindow(state != BST_CHECKED);
	m_ctrlScaleHigh.EnableWindow(state != BST_CHECKED);
	if (mThreadDisplay) {
		mThreadDisplay->SetManualAdjust(state != BST_CHECKED);
	}
}

void PvSimpleUISampleDlg::OnBnClickedOneshotadj()
{
	if (mThreadDisplay) {
		mThreadDisplay->AdjustOnce();
		m_bAdjustOnce = true;
	}
}

void PvSimpleUISampleDlg::OnEnKillfocusSclo()
{
    CTemperature T;
	USHORT v;

    UpdateData(TRUE);
	TRACE("%.1f\r\n", m_fScaleLo);
    if (m_iPresentation == PRES_CELSIUS) {
        T.setUnit(CTemperature::Celsius);
    }
    else if (m_iPresentation == PRES_FAHRENHEIT) {
        T.setUnit(CTemperature::Fahrenheit);
    }
    else if (m_iPresentation == PRES_KELVIN) {
        T.setUnit(CTemperature::Kelvin);
    }

    if (mThreadDisplay) 
	{
        if (m_IRFormat == IRFMT_SIGNAL) 
        {
            if (m_iPresentation == PRES_SIGNAL)
            {
                v = (USHORT)m_fScaleLo;
            }
            else
            {
                T.Set(m_fScaleLo);
       			if (m_CamType == CAM_TYPE_ATAU)
                {
                    if (m_bMeasCapable)
                       v = TauToSignal((float)(T.Value(CTemperature::Celsius)));
                }
                else 
                {
                    if (m_bMeasCapable)
                        v = tempToImg(T.Value(CTemperature::Kelvin));
                    else
		    	       v = (USHORT)m_fScaleLo;
                }
            }
        }
        else if (m_IRFormat == IRFMT_TEMPLIN_LORES) 
        {
            if (m_iPresentation == PRES_SIGNAL)
            {
                v = (USHORT)m_fScaleLo;
            }
            else
            {
                T.Set(m_fScaleLo);
                if (m_CamType == CAM_TYPE_ATAU && m_bMeasCapable) {
                    v = (USHORT)(T.Value(CTemperature::Kelvin) * 2.5f);
                }
                else
                    v = (USHORT)(T.Value(CTemperature::Kelvin) * 10.0f);
            }
        }
        else if (m_IRFormat == IRFMT_TEMPLIN_HIRES) 
        {
            if (m_iPresentation == PRES_SIGNAL)
            {
                v = (USHORT)m_fScaleLo;
            }
            else
            {
                T.Set(m_fScaleLo);
                if (m_CamType == CAM_TYPE_ATAU && m_bMeasCapable)
                    v = (USHORT)(T.Value(CTemperature::Kelvin) * 25.0f);
                else
                    v = (USHORT)(T.Value(CTemperature::Kelvin) * 100.0f);
            }
        }

		mThreadDisplay->setScaleLo(v);
	}
}

void PvSimpleUISampleDlg::OnEnKillfocusSchi()
{
    CTemperature T;
	USHORT v;

    UpdateData(TRUE);
	TRACE("%.1f\r\n", m_fScaleHi);

    if (m_iPresentation == PRES_CELSIUS) {
        T.setUnit(CTemperature::Celsius);
    }
    else if (m_iPresentation == PRES_FAHRENHEIT) {
        T.setUnit(CTemperature::Fahrenheit);
    }
    else if (m_iPresentation == PRES_KELVIN) {
        T.setUnit(CTemperature::Kelvin);
    }

	if (mThreadDisplay) 
	{
        if (m_IRFormat == IRFMT_SIGNAL) 
        {
            if (m_iPresentation == PRES_SIGNAL)
            {
                v = (USHORT)m_fScaleHi;
            }
            else
            {
                T.Set(m_fScaleHi);
                if (m_CamType == CAM_TYPE_ATAU)
                {
                    v = TauToSignal((float)(T.Value(CTemperature::Celsius)));
                }
                else
                {
                    if (m_bMeasCapable)
                        v = tempToImg(T.Value(CTemperature::Kelvin));
                    else
    	    		    v = (USHORT)m_fScaleHi;
                }
            }
        }
        else if (m_IRFormat == IRFMT_TEMPLIN_LORES) 
        {
            if (m_iPresentation == PRES_SIGNAL)
            {
                v = (USHORT)m_fScaleHi;
            }
            else
            {
                T.Set(m_fScaleHi);
                // Temperature linear mode - low resolution
                if (m_CamType == CAM_TYPE_ATAU && m_bMeasCapable) 
                    v = (USHORT)(T.Value(CTemperature::Kelvin) * 2.5f);
                else
                    v = (USHORT)(T.Value(CTemperature::Kelvin) * 10.0f);
            }
        }
        else if (m_IRFormat == IRFMT_TEMPLIN_HIRES) 
        {
            if (m_iPresentation == PRES_SIGNAL)
            {
                v = (USHORT)m_fScaleHi;
            }
            else
            {
                T.Set(m_fScaleHi);
                // Temperature linear mode - high resolution
                // Temperature linear mode - low resolution
                if (m_CamType == CAM_TYPE_ATAU && m_bMeasCapable) 
                    v = (USHORT)(T.Value(CTemperature::Kelvin) * 25.0f);
                else
                    v = (USHORT)(T.Value(CTemperature::Kelvin) * 100.0f);
            }
        }

		mThreadDisplay->setScaleHi(v);
	}
}

void PvSimpleUISampleDlg::OnBnClickedNuc()
{
	PvGenCommand *lNUC = dynamic_cast<PvGenCommand *>( mDevice->GetParameters()->Get( "NUCAction" ) );
	if (lNUC) {
		lNUC->Execute();
	}
}

void PvSimpleUISampleDlg::TrigCheck()
{
	if (m_CamType == CAM_TYPE_T1K)
	{
        DWORD state, trig1;
        CString s;
		trig1 = mThreadDisplay->getTrigCount(1);
        state = mThreadDisplay->getLineState(1);
		s.Format(_T("%d"), trig1);
    	m_ctrlTrigCount1.SetWindowText(s);
        m_ctrlPort1State.SetWindowText(state == 0 ? _T("Low") : _T("High"));
    }
}

void PvSimpleUISampleDlg::AlarmCheck()
{
	double dblValue;
	if (!m_bAlarmActive)
		return;

	if (!mAcquiringImages)
		return;

	// Check if alarm condition is fulfilled
	if (mThreadDisplay)
	{
		USHORT v;

		// Get correct value to check alarm on
		if (m_iAlarmFunc == ALARM_FUNC_SPOT)
			v = mThreadDisplay->getCenterSpot();
		else
		{
			if (m_iAlarmResult == ALARM_RES_AVG)
				v = mThreadDisplay->getAreaAvg();
			else if (m_iAlarmResult == ALARM_RES_MIN)
				v = mThreadDisplay->getAreaMin();
			else
				v = mThreadDisplay->getAreaMax();
		}

		if (m_IRFormat == IRFMT_SIGNAL)
        {
            // Signal mode
    		if (m_CamType == CAM_TYPE_ATAU && m_bMeasCapable)
                dblValue = TauToTemp(v) - 273.15f;
            else 
            {
                if (m_bMeasCapable) 
                {
                    CTemperature T(CTemperature::Celsius);
                    T = imgToTemp(v);
                    dblValue = T.Value();
                }
                else
    			    dblValue = (double)(v / 10.0f) - 273.15f;
            }
        }
		else if (m_IRFormat == IRFMT_TEMPLIN_LORES)
        {
    		if (m_CamType == CAM_TYPE_ATAU && m_bTLUTCapable)
			    dblValue = (double)(v * 0.4) - 273.15f;
            else
			    dblValue = (double)(v / 100.0f) - 273.15f;
        }
		else if (m_IRFormat == IRFMT_TEMPLIN_HIRES)
		    dblValue = (double)(v * 0.04) - 273.15f;

		if (m_iAlarmCondition == ALARM_COND_ABOVE)
			m_bAlarmTrigged = (dblValue > m_dblAlarmValue);
		else 
			m_bAlarmTrigged = (dblValue < m_dblAlarmValue);

		if (m_bAlarmTrigged) 
		{
			m_ctrlAlarmState.SetWindowText(_T("ALARM"));
			// Do specified alarm action
			if (m_bFlash) 
            {
				if ((GetTickCount() - m_bFlashTS) > 500) 
				{
					COLORREF c = m_ctrlAlarmState.GetBGColor();
					m_bFlashTS = GetTickCount();
					if (c == RED)
						m_ctrlAlarmState.SetBGColor(YELLOW, TRUE);
					else
						m_ctrlAlarmState.SetBGColor(RED, TRUE);
				}
			}
			else
				m_ctrlAlarmState.SetBGColor(RED, TRUE);

    		if (m_CamType == CAM_TYPE_ATAU)
            {
    			if (m_DigOut > 0) 
                {
	    			PvGenBoolean *lDigState = dynamic_cast<PvGenBoolean *>( mDevice->GetParameters()->Get( "UserOutputValue" ) );
                    if (lDigState) {
	    				if ((GetTickCount() - m_bAlarmTS) > m_PulseTime)
                            lDigState->SetValue( false ); // DEASSERTED
                        else
                            lDigState->SetValue( true ); // ASSERTED
                    }
                }
			}
            else
            {
    			if (m_DigOut > 0) {
                    EnterCriticalSection( &m_csPort );
	    			PvGenInteger *lDigPort = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "CurrentPort" ) );
		    		PvGenEnum *lDigState = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "CurrentIOState" ) );
			    	if (lDigPort) lDigPort->SetValue( m_DigOut + 1 );
				    if (lDigState)
    				{
	    				if ((GetTickCount() - m_bAlarmTS) > m_PulseTime)
		    				lDigState->SetValue( 0 ); // DEASSERTED
			    		else
				    		lDigState->SetValue( 1 ); // ASSERTED
    				}	
                    LeaveCriticalSection( &m_csPort );
                }
            }
		}
		else 
		{
			m_ctrlAlarmState.SetWindowText(_T("Waiting"));
			m_ctrlAlarmState.SetBGColor(GREEN, TRUE);
			m_bAlarmTS = GetTickCount();

    		if (m_CamType == CAM_TYPE_ATAU)
            {
    			if (m_DigOut > 0) {
	    			PvGenBoolean *lDigState = dynamic_cast<PvGenBoolean *>( mDevice->GetParameters()->Get( "UserOutputValue" ) );
				    if (lDigState) lDigState->SetValue( false );
			    }
            }
            else
            {
    			if (m_DigOut > 0) {
                    EnterCriticalSection( &m_csPort );
	    			PvGenInteger *lDigPort = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "CurrentPort" ) );
		    		PvGenEnum *lDigState = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "CurrentIOState" ) );
			    	if (lDigPort) lDigPort->SetValue( m_DigOut + 1 );
				    if (lDigState) lDigState->SetValue( 0 ); // DEASSERTED
                    LeaveCriticalSection( &m_csPort );
			    }
            }
		}
	}
}

void PvSimpleUISampleDlg::OnBnClickedAlarmSetup()
{
	CAlarmSetupDlg Dlg;

	Dlg.m_bActive = m_bAlarmActive;
	Dlg.m_AlarmFunc = m_iAlarmFunc;
	Dlg.m_AlarmResult = m_iAlarmResult;
	Dlg.m_iCondition = m_iAlarmCondition;
	Dlg.m_dwPulseTime = m_PulseTime;
	Dlg.m_DigOut = m_DigOut;
	Dlg.m_AlarmValue = m_dblAlarmValue;
	Dlg.m_bFlash = m_bFlash;
	if (m_CamType == CAM_TYPE_ATAU)
        Dlg.m_iDigPorts = 1;
    else
        Dlg.m_iDigPorts = 2;

	if (IDOK == Dlg.DoModal()) 
	{
		m_bAlarmActive = Dlg.m_bActive;
		m_iAlarmFunc = Dlg.m_AlarmFunc;
		m_iAlarmResult = Dlg.m_AlarmResult;
		m_iAlarmCondition = Dlg.m_iCondition;
		m_PulseTime = Dlg.m_dwPulseTime;
		m_DigOut = Dlg.m_DigOut;
		m_dblAlarmValue = Dlg.m_AlarmValue;
		m_bFlash = Dlg.m_bFlash;

		if (m_bAlarmActive)
		{
    		m_bAlarmTS = m_bFlashTS = GetTickCount();
			m_ctrlAlarmState.SetWindowText(_T("Waiting"));
			m_ctrlAlarmState.SetBGColor(GREEN, TRUE);
        	if (m_CamType != CAM_TYPE_ATAU)
            {
                EnterCriticalSection( &m_csPort );
    			if (m_DigOut > 0) {
	    			// Configure digital output port
		    		PvGenInteger *lDigPort = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "CurrentPort" ) );
			    	PvGenEnum *lDigCfg = dynamic_cast<PvGenEnum *>( mDevice->GetParameters()->Get( "CurrentIOConfig" ) );
				    if (lDigPort) lDigPort->SetValue( m_DigOut + 1 );
				    if (lDigCfg) lDigCfg->SetValue( 0 ); // General purpose output port
			    }
                LeaveCriticalSection( &m_csPort );
            }
		}
		else {
			m_ctrlAlarmState.SetWindowText(_T("Inactive"));
			m_ctrlAlarmState.SetBGColor(WHITE, TRUE);
		}
	}
}

BOOL PvSimpleUISampleDlg::PreTranslateMessage(MSG* pMsg) 
{ 
    if (pMsg->message == WM_MOUSEMOVE)
	{
		if (pMsg->hwnd == (HWND)mDisplay.GetHandle())
        {
    		int x = GET_X_LPARAM(pMsg->lParam);
	    	int y = GET_Y_LPARAM(pMsg->lParam);

            if (mAcquiringImages) {
                m_bFlyingSpotActive = true;
	    		::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_CROSS));
            }

		    if (mThreadDisplay)
			    mThreadDisplay->setSpot(CPoint(x,y));
        }
        else 
        {
            if (m_bFlyingSpotActive) {
        		::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
                m_bFlyingSpotActive = false;
            }
        }
	}

    if (pMsg->message == WM_KEYDOWN && 
        pMsg->wParam == VK_RETURN && 
		GetFocus() == FromHandle(m_ctrlScaleLow.m_hWnd)) 
    { 
        // handle return pressed in edit control 
		OnEnKillfocusSclo();
		return TRUE;
    } 

    if (pMsg->message == WM_KEYDOWN && 
        pMsg->wParam == VK_RETURN && 
		GetFocus() == FromHandle(m_ctrlScaleHigh.m_hWnd)) 
    { 
        // handle return pressed in edit control 
		OnEnKillfocusSchi();
		return TRUE;
    } 

	return CDialog::PreTranslateMessage(pMsg); // FALSE;
} 

void PvSimpleUISampleDlg::OnBnClickedAreaSetup()
{
	CAreaSetupDlg dlg;
	CPoint ul;
	CSize sz;

	if (mThreadDisplay)
	{
		ul = mThreadDisplay->getAreaUL();
		sz = mThreadDisplay->getAreaSize();
	}
	
	dlg.m_X = ul.x;
	dlg.m_Y = ul.y;
	dlg.m_Width  = sz.cx;
	dlg.m_Height = sz.cy;
    dlg.m_bActive = m_bAreaActive;

	if (IDOK == dlg.DoModal()) 
	{
		ul.x = dlg.m_X;
		ul.y = dlg.m_Y;
		sz.cx = dlg.m_Width;
		sz.cy = dlg.m_Height;
        m_bAreaActive = dlg.m_bActive;
        if (mThreadDisplay) {
			mThreadDisplay->SetArea(ul, sz);
            mThreadDisplay->setAreaActive(m_bAreaActive);
        }

        if (!m_bAreaActive) {
            CString s;
            s.Empty();
            m_ctrlAreaAvg.SetWindowText(s);
	    	m_ctrlAreaMin.SetWindowText(s);
		    m_ctrlAreaMax.SetWindowText(s);
        }
	}
}
void PvSimpleUISampleDlg::OnBnClickedAbout()
{
	AboutDlg dlg;
	dlg.DoModal();
}

void PvSimpleUISampleDlg::OnBnClickedLog()
{
	CLogDlg dlg;
	dlg.m_bLog = m_bLogging;
    dlg.m_Interval = m_iLogInterval / 2;
	if (IDOK == dlg.DoModal()) {
		m_bLogging = dlg.m_bLog;
        m_iLogInterval = dlg.m_Interval * 2;
	}
}

// Perform a one-point calibration. Calculate a new offset based on a known
// temperature (from a blackbody radiator for example). Please observe that
// new offset value is NOT made persistent. In order to make the offset persistent,
// you need to submit the GeniCAM command SensorSetDefaults
void PvSimpleUISampleDlg::OnBnClickedCalibrate()
{
    CCalibDlg Dlg;
    Dlg.m_BBTemp = 25;
	if (IDOK == Dlg.DoModal()) 
	{
        double O;
        if (mThreadDisplay) {
            CString s;
            float K = (float)Dlg.m_BBTemp + 273.15f;
            O = (double)mThreadDisplay->getCenterSpot() - ((double)m_R / (exp(m_B/K) - m_F));
            PvGenParameterArray *lGenDevice = mDevice->GetParameters();
            // Try with OInt feature register (O register is read only)
           	PvGenInteger *lOInt = dynamic_cast<PvGenInteger *>( lGenDevice->Get( "OInt" ) );
            if (lOInt)
            {
               m_O = O;
               lOInt->SetValue((int64_t)O);
               s.Format(_T("New calculated offset = %.2f"), m_O);
               AfxMessageBox(s);
            }
            else 
            {
               AfxMessageBox(_T("Unable to set offset value"));
            }
        }
    }
}

double PvSimpleUISampleDlg::imgToPow(long lPixval)
{
    double pow;

    pow = (lPixval - m_J0) / m_J1;
    
    return (pow);
}

USHORT PvSimpleUISampleDlg::tempToImg(double dKelvin)
{
    USHORT pixVal;
    double tmp;
    
    tmp = tempToObjSig(dKelvin);
    tmp = objSigToPow(tmp);
    pixVal = powToImg(tmp);

    return pixVal;
}

double PvSimpleUISampleDlg::objSigToPow(double dObjSig)
{
#define POW_OVERFLOW   100000.0  
    double p;
    
    if (m_K1 > 0.0)
        p = (dObjSig + m_K2)/m_K1;
    else
        p = POW_OVERFLOW;

    return (p);
}

USHORT PvSimpleUISampleDlg::powToImg(double dPow)
{
    // Convert a power equivalent pixel value to mapped FPA pixel value  
    long rpix;

    rpix = (long)(m_J1 * dPow + m_J0);

    return (USHORT) rpix;    
}

double PvSimpleUISampleDlg::tempToObjSig(double dblKelvin)
{
    double objSign = 0.0;
    double dbl_reg = dblKelvin;

    // objSign = R / (exp(B/T) - F)
 
    if (dbl_reg > 0.0) {
 
        dbl_reg = m_B / dbl_reg; 

        if (dbl_reg < EXP_SAFEGUARD) {
            dbl_reg = exp(dbl_reg); 
         
            if (m_F <= 1.0) {
                if ( dbl_reg < ASY_SAFEGUARD )
                    dbl_reg = ASY_SAFEGUARD; // Don't get above a R/(1-F)
                                             // (horizontal) asymptote
            }                      
            else 
            {
                // F > 1.0
                if ( dbl_reg < m_F*ASY_SAFEGUARD )
                    dbl_reg = m_F*ASY_SAFEGUARD;
                    // Don't get too close to a B/ln(F) (vertical) asymptote
            }
 
            objSign = m_R/(dbl_reg - m_F);
        }
    }
    
    return( objSign);
}

double PvSimpleUISampleDlg::powToObjSig(double dPow)
{
    return (m_K1 * dPow - m_K2);
}

CTemperature PvSimpleUISampleDlg::objSigToTemp(double dObjSig)
{
    double dbl_reg, tmp;
    CTemperature Tkelvin = 0.0;

    // Tkelvin = B /log(R / objSign + F)

    if (dObjSig > 0.0)
    {
        dbl_reg = m_R / dObjSig + m_F;
     
        if (m_F <= 1.0) {
            if (dbl_reg < ASY_SAFEGUARD)
                dbl_reg = ASY_SAFEGUARD; // Don't get above a R/(1-F)
                                         // (horizontal) asymptote
        }                       
        else { // if (m_F > 1.0)

            tmp = m_F * ASY_SAFEGUARD;
            if (dbl_reg < tmp)
                dbl_reg = tmp;
                // Don't get too close to a B/ln(F) (vertical) asymptote
        }
        Tkelvin = m_B / quicklog(dbl_reg);        
    }

    return (Tkelvin);
}

CTemperature PvSimpleUISampleDlg::imgToTemp(long lPixval)
{
    double tmp;
    CTemperature K;

    tmp = imgToPow(lPixval);
    //tmp = clipPow(tmp,pValState);
    tmp = powToObjSig(tmp);
    K = objSigToTemp(tmp);

    return (K);
}

double PvSimpleUISampleDlg::doCalcK1(void)
{
    double dblVal = 1.0;

    dblVal = m_AtmTao * m_Emissivity * m_ExtOptTransm;

    if (dblVal > 0.0)
       dblVal = 1/dblVal;

    return (dblVal);
}

double PvSimpleUISampleDlg::doCalcK2(double dAmbObjSig, 
                                     double dAtmObjSig, 
                                     double dExtOptTempObjSig)
{
    double emi;
    double temp1 = 0.0;
    double temp2 = 0.0;
    double temp3 = 0.0;
    
    emi  = m_Emissivity;

    if (emi > 0.0)
    {
        temp1 = (1.0 - emi)/emi * dAmbObjSig;

        if (m_AtmTao > 0.0) {
            temp2 = (1.0 - m_AtmTao)/(emi*m_AtmTao)* dAtmObjSig;

           if (m_ExtOptTransm > 0.0 && m_ExtOptTransm < 1.0) {
               temp3 = (1.0 - m_ExtOptTransm)/
                  (emi*m_AtmTao*m_ExtOptTransm)* dExtOptTempObjSig;
           }
        }
    }  
    
    return (temp1 + temp2 + temp3);
}

double PvSimpleUISampleDlg::doCalcAtmTao(void)
{
    double tao, dtao;
    double H, T, sqrtD, X, a1, b1, a2, b2;
    double sqrtH2O;
    double TT;
    double a1b1sqH2O, a2b2sqH2O, exp1, exp2;
    CTemperature C(CTemperature::Celsius);
    
#define H2O_K1 +1.5587e+0
#define H2O_K2 +6.9390e-2
#define H2O_K3 -2.7816e-4
#define H2O_K4 +6.8455e-7
#define TAO_TATM_MIN -30.0
#define TAO_TATM_MAX  90.0
#define TAO_SQRTH2OMAX 6.2365
#define TAO_COMP_MIN 0.400
#define TAO_COMP_MAX 1.000

    H = m_RelHum;                    
    C = m_AtmTemp;
    T = C.Value();        // We need Celsius to use constants defined above
    sqrtD = sqrt(m_ObjectDistance);
    X  = m_X;
    a1 = m_alpha1;     
    b1 = m_beta1;
    a2 = m_alpha2;
    b2 = m_beta2; 
    
    if (T < TAO_TATM_MIN)
       T = TAO_TATM_MIN;
    else if (T > TAO_TATM_MAX)
       T = TAO_TATM_MAX;
       
    TT = T*T;
    
    sqrtH2O = sqrt(H*exp(H2O_K1 + H2O_K2*T + H2O_K3*TT + H2O_K4*TT*T));
    
    if ( sqrtH2O > TAO_SQRTH2OMAX )
       sqrtH2O = TAO_SQRTH2OMAX;
       
    a1b1sqH2O = (a1+b1*sqrtH2O);
    a2b2sqH2O = (a2+b2*sqrtH2O);
    exp1    = exp(-sqrtD*a1b1sqH2O);
    exp2    = exp(-sqrtD*a2b2sqH2O); 
    
    tao     = X*exp1 + (1-X)*exp2;
    dtao    = -(a1b1sqH2O*X*exp1+a2b2sqH2O*(1-X)*exp2); 
    // The real D-derivative is also divided by 2 and sqrtD.
    // Here we only want the sign of the slope! */
                                                         
    if (tao < TAO_COMP_MIN)
        tao = TAO_COMP_MIN;      // below min value, clip

    else if (tao > TAO_COMP_MAX)
    {
        // check tao at 1 000 000 m dist
        tao  = X*exp(-(1.0E3)*a1b1sqH2O)+(1.0-X)*exp(-(1.0E3)*a2b2sqH2O);

      if ( tao > 1.0 )    // above max, staying up, assume \/-shape
          tao = TAO_COMP_MIN;
      else
          tao = TAO_COMP_MAX; // above max, going down, assume /\-shape
    }
    else if ( dtao > 0.0 && m_ObjectDistance > 0.0)
        tao = TAO_COMP_MIN;   // beween max & min, going up, assume \/

    // else between max & min, going down => OK as it is, ;-)

    return( tao);
}

void PvSimpleUISampleDlg::doUpdateCalcConst(void)
{
    m_AtmTao = doCalcAtmTao();

    m_K1 = doCalcK1();

    m_K2 = doCalcK2(tempToObjSig(m_AmbTemp),
                    tempToObjSig(m_AtmTemp),
                    tempToObjSig(m_ExtOptTemp));
}

void PvSimpleUISampleDlg::updateCalcParams()
{
    if (!m_bMeasCapable)
        return;

    if (m_CalcCheckNeeded <= 0)
        return;

	if (m_CamType == CAM_TYPE_ATAU)
    {
        // Update RBF0
        PvGenInteger *lR = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "R" ) );
       	PvGenFloat *lB = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "B" ) );
        PvGenFloat *lF = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "F" ) );
        PvGenFloat *lO = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "O" ) );

        if (lR && lB && lF && lO)
        {
           PvResult lResult;
           int64_t tmpR = 0;
           lResult = lR->GetValue(tmpR);
           m_R = (int)tmpR;
           lB->GetValue(m_B);
           lF->GetValue(m_F);
           lO->GetValue(m_O);
           TRACE("R=%d\r\n", m_R);
           TRACE("B=%.1f\r\n", m_B);
           TRACE("F=%.1f\r\n", m_F);
           TRACE("O=%.1f\r\n", m_O);
        }

        doUpdateCalcConst();
    }
    else
    {
        // Update RBF
       	PvGenFloat *lR = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "R" ) );
   	    PvGenFloat *lB = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "B" ) );
       	PvGenFloat *lF = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "F" ) );

        if (lR && lB && lF)
        {
           PvResult lResult;
           double tmpR = 0;
           lR->GetValue(tmpR);
           m_R = (int)tmpR;
           lB->GetValue(m_B);
           lF->GetValue(m_F);
           TRACE("R=%d\r\n", m_R);
           TRACE("B=%.1f\r\n", m_B);
           TRACE("F=%.1f\r\n", m_F);
        }

        // Update Gain and Offset (J1 and J0)
        PvGenInteger *lJ0 = dynamic_cast<PvGenInteger *>( mDevice->GetParameters()->Get( "J0" ) );
        PvGenFloat *lJ1 = dynamic_cast<PvGenFloat *>( mDevice->GetParameters()->Get( "J1" ) );
        if (lJ0 && lJ1) 
        {
            int64_t tmp;
            lJ0->GetValue(tmp);
            m_J0 = (ULONG)tmp;
            lJ1->GetValue(m_J1);
            TRACE("J0=%d\r\n", m_J0);
            TRACE("J1=%.1f\r\n", m_J1);
         }

         doUpdateCalcConst();
    }

    m_CalcCheckNeeded--;
}

void PvSimpleUISampleDlg::OnBnClickedObjpars()
{
    CEditObjParsDlg dlg;
    CTemperature T(CTemperature::Kelvin);

    // Init dialog values
    T.Set(m_AmbTemp);
    dlg.m_AmbTemp = T.Value(CTemperature::Celsius);
    dlg.m_Emissivity = m_Emissivity;
    dlg.m_ObjectDistance = m_ObjectDistance;
    dlg.m_RelHum = m_RelHum;
    T.Set(m_AtmTemp);
    dlg.m_AtmTemp = T.Value(CTemperature::Celsius);
    T.Set(m_ExtOptTemp);
    dlg.m_ExtOptTemp = T.Value(CTemperature::Celsius);
    dlg.m_ExtOptTransm = m_ExtOptTransm;

    if (dlg.DoModal() == IDOK)
    {
        m_Emissivity = dlg.m_Emissivity;
        T.Set(dlg.m_AmbTemp, CTemperature::Celsius);
        m_AmbTemp = T.Value(CTemperature::Kelvin);
        m_ObjectDistance = dlg.m_ObjectDistance;
        m_RelHum = dlg.m_RelHum / 100.0f;
        T.Set(dlg.m_AtmTemp, CTemperature::Celsius);
        m_AtmTemp = T.Value(CTemperature::Kelvin);
        m_ExtOptTransm = dlg.m_ExtOptTransm;
        T.Set(dlg.m_ExtOptTemp, CTemperature::Celsius);
        m_ExtOptTemp = T.Value(CTemperature::Kelvin);
        UpdateDeviceOP();
   }
}

void PvSimpleUISampleDlg::UpdateDeviceOP()
{
    PvGenParameterArray *lGenDevice;
    lGenDevice = mDevice->GetParameters();

    if (m_CamType != CAM_TYPE_ATAU)
    {
        // Update device object parameters
        PvGenFloat *lEmissivity = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ObjectEmissivity" ) );
        PvGenFloat *lExtOptTransm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ExtOpticsTransmission" ) );
        PvGenFloat *lRelHum = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "RelativeHumidity" ) );
        PvGenFloat *lDist = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ObjectDistance" ) );
        PvGenFloat *lAmb = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ReflectedTemperature" ) );
        PvGenFloat *lAtm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "AtmosphericTemperature" ) );
        PvGenFloat *lEOT = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ExtOpticsTemperature" ) );
        
        if (lEmissivity && lExtOptTransm && lRelHum && lDist && lAmb && lAtm && lEOT)
        {
            lEmissivity->SetValue(m_Emissivity);
            lExtOptTransm->SetValue(m_ExtOptTransm);
            lRelHum->SetValue(m_RelHum);
            lDist->SetValue(m_ObjectDistance);
            lAmb->SetValue(m_AmbTemp);
            lAtm->SetValue(m_AtmTemp);
            lEOT->SetValue(m_ExtOptTemp);
        }

        if (m_bMeasCapable)
            doUpdateCalcConst();
    }
    else 
    {
        // Update device object parameters
        PvGenFloat *lEmissivity = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ObjectEmissivity" ) );
        PvGenFloat *lAmb = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "ReflectedTemperature" ) );
        PvGenFloat *lAtm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "AtmosphericTemperature" ) );
        PvGenFloat *lEOT = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "WindowTemperature" ) );
        PvGenFloat *lExtOptTransm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "WindowTransmission" ) );

        if (lEmissivity && lAmb && lAtm && lEOT && lExtOptTransm)
        {
            lEmissivity->SetValue(m_Emissivity);
            lAmb->SetValue(m_AmbTemp);
            lAtm->SetValue(m_AtmTemp);
            lEOT->SetValue(m_ExtOptTemp);
            lExtOptTransm->SetValue(m_ExtOptTransm);
        }

        if (m_bMeasCapable) 
        {
            PvGenFloat *lAtmTransm = dynamic_cast<PvGenFloat *>( lGenDevice->Get( "AtmosphericTransmission" ) );
            doUpdateCalcConst();
            if (lAtmTransm)
            {
                lAtmTransm->SetValue(m_AtmTao);
            }
        }
    }
}
