// *****************************************************************************
//
//     Copyright (c) 2007, Pleora Technologies Inc., All rights reserved.
//
// *****************************************************************************

#pragma once

#include "afxwin.h"

#include "Thread.h"
#include "Resource.h"

#include <PvDeviceFinderWnd.h>
#include <PvDevice.h>
#include <PvGenParameter.h>
#include <PvGenBrowserWnd.h>
#include <PvBuffer.h>
#include <PvStream.h>
#include <PvPipeline.h>
#include <PvDisplayWnd.h>
#include <PvAcquisitionStateManager.h>

#include <PvDeviceGEV.h>
#include <PvDeviceInfoGEV.h>
#include <PvDeviceInfoU3V.h>
#include <PvStreamGEV.h>

#include "afxcmn.h"
#include "temperature.hpp"

#define WM_IMAGEDISPLAYED ( WM_USER + 0x4431 )
#define WM_DISCONNECT ( WM_USER + 0x4432 )
#define WM_STREAM ( WM_USER + 0x4433 )
#define WM_STOPSTREAMING ( WM_USER + 0x4434 )
#define WM_LINKDISCONNECTED ( WM_USER + 0x4438 )
#define WM_LINKRECONNECTED ( WM_USER + 0x4439 )
#define WM_ACQUISITIONSTATECHANGED ( WM_USER + 0x4440 )
#define WM_UPDATESOURCE ( WM_USER + 0x4441 )
#define WM_UPDATESOURCES ( WM_USER + 0x4442 )
#define WM_UPDATEACQUISITIONMODE ( WM_USER + 0x4443 )

#include "ColorButton.h"

#define CAM_TYPE_A3XX 0
#define CAM_TYPE_A6XX 1
#define CAM_TYPE_ATAU 2
#define CAM_TYPE_T1K  3

#define PRES_SIGNAL     0
#define PRES_CELSIUS    1
#define PRES_FAHRENHEIT 2
#define PRES_KELVIN     3

#define IRFMT_SIGNAL        0
#define IRFMT_TEMPLIN_LORES 1
#define IRFMT_TEMPLIN_HIRES 2

class PvGenStateStack;

class PvSimpleUISampleDlg : public CDialog, PvGenEventSink, PvDeviceEventSink, PvAcquisitionStateEventSink
{
public:

	PvSimpleUISampleDlg( CWnd *pParent = NULL );
	virtual ~PvSimpleUISampleDlg();

	enum { IDD = IDD_PVSIMPLEUISAMPLE };

	void StartStreaming();
	void StopStreaming();

    bool IsAcquiringImages() const { return mAcquiringImages; }
    void   AlarmCheck();
    void   TrigCheck();
    static bool IsMultiSourceTransmitter( PvDevice *aDevice );

protected:

	HICON m_hIcon;

	BOOL PreTranslateMessage(MSG* pMsg);
	virtual void DoDataExchange(CDataExchange* pDX);
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnGetMinMaxInfo( MINMAXINFO *lpMMI );
	afx_msg void OnBnClickedDeviceButton();
	afx_msg void OnBnClickedLinkButton();
	afx_msg void OnBnClickedStreamparamsButton();
	afx_msg void OnBnClickedConnectButton();
	afx_msg void OnBnClickedDisconnectButton();
	afx_msg void OnClose();
	afx_msg void OnBnClickedStart();
	afx_msg void OnBnClickedStop();
	afx_msg void OnBnClickedDeviceEvents();
	afx_msg void OnMove(int x, int y);
    LRESULT OnAcquisitionStateChanged(WPARAM wParam, LPARAM lParam);
    LRESULT OnLinkReconnected( WPARAM wParam, LPARAM lParam );
    LRESULT OnLinkDisconnected( WPARAM wParam, LPARAM lParam );
    LRESULT OnStopStreaming( WPARAM wParam, LPARAM lParam );
    LRESULT OnDisconnect( WPARAM wParam, LPARAM lParam );
    LRESULT OnStream( WPARAM wParam, LPARAM lParam );

    DECLARE_MESSAGE_MAP()

    void EnableInterface();
	void EnumCases();

	void Connect( const PvDeviceInfo *aDI );
	void Disconnect();
    void StartAcquisition();
    void StopAcquisition();

    void ShowGenWindow( PvGenBrowserWnd **aWnd, PvGenParameterArray *aParams, const CString &aTitle );
	void CloseGenWindow( PvGenBrowserWnd **aWnd );
    void PushSource( PvGenStateStack *aStack );

    // PvGenEventSink implementation
	void OnParameterUpdate( PvGenParameter *aParameter );

    // PvDeviceEventSink implementation
    void OnLinkDisconnected( PvDevice *aDevice );
    void OnLinkReconnected( PvDevice *aDevice );
    void OnEvent( PvDevice *aDevice, uint16_t aEventID, uint16_t aChannel, uint16_t aBlockID, 
        uint64_t aTimestamp, const void *aData, uint32_t aDataLength );

    // PvAcquisitionStateChanged implementation
    void OnAcquisitionStateChanged(PvDevice* aDevice, 
                                   PvStream* aStream,
                                   uint32_t aSource,
                                   PvAcquisitionState aState);

    CRect mCrt;
	BOOL mNeedInit;
    BOOL mRecovery;

	CButton mPlayButton;
	CButton mStopButton;

	CEdit mIPEdit;
	CEdit mMACEdit;
	CEdit mManufacturerEdit;
	CEdit mModelEdit;
	CEdit mNameEdit;

	PvDevice* mDevice;
    PvStream* mStream;
    PvPipeline* mPipeline;
    PvString mConnectionID;

    bool mAcquiringImages;
    bool m_bRecording;
	bool m_bLogging;
	int  m_iLogCount;
    int  m_iLogInterval;
    CRITICAL_SECTION    m_csPort;

	Thread*             mThreadDisplay;
	PvGenBrowserWnd*    mDeviceWnd;
	PvGenBrowserWnd*    mCommunicationWnd;
	PvGenBrowserWnd*    mStreamParametersWnd;
	PvDisplayWnd        mDisplay;
    PvAcquisitionStateManager* mAcquisitionStateManager;
	
	int					m_Width;
	int					m_Height;
	int					mpvPixelFormat;
    int                 miPixelSize;
	bool                mbWorkaround;
    bool                m_bSingleRange;
    int                 m_iPresentation;
    int                 m_WindowingMode;

 	int                 m_CamType;
	UINT                m_Timer;
	bool                m_bAutoAdjust;
	bool                m_bAdjustOnce;
	int					m_IRFormat;
    bool                m_bFlyingSpotActive;
    bool				m_bAreaActive;

    bool				m_bAlarmActive;
	int					m_iAlarmFunc;
	int					m_iAlarmResult;
	int					m_iAlarmCondition;
	DWORD				m_PulseTime;
	int					m_DigOut;
	double				m_dblAlarmValue;
	BOOL				m_bFlash;
	BOOL				m_bFlashCount;
	DWORD				m_bFlashTS;
	BOOL				m_bAlarmTrigged;
	DWORD				m_bAlarmTS;
    bool                m_bFrameRateControlEnabled;
	int					m_CalcCheckNeeded;

public:
    // Tau Planck constants
    int                 m_R;
    double              m_B;
    double              m_F;
    double              m_O;
    double              m_WT;
    double              m_W4WT;
    ULONG               m_J0; // Global offset
    double              m_J1; // Global gain
    double              m_K1;
    double              m_K2;
    // Object parameters
    double              m_Emissivity;
    double              m_ObjectDistance;
    double              m_AtmTemp;
    double              m_AmbTemp;
    double              m_AtmTao;
    double              m_ExtOptTransm;
    double              m_ExtOptTemp;
    double              m_RelHum;                    
    // Spectral response parameters
    double              m_X;
    double              m_alpha1;     
    double              m_alpha2;
    double              m_beta1;
    double              m_beta2; 

protected:
    bool                m_bMeasCapable;
    bool                m_bTLUTCapable; // Capacity to generate temperature linear images
    // Tau frame rate fast or slow
    bool                m_bFast;

    void   updateCalcParams();
    float  TauToTemp(USHORT sig);
    USHORT TauToSignal(float C);
    double imgToPow(long lPixval);
    double powToObjSig(double dPow);
    double tempToObjSig(double dblKelvin);
    USHORT tempToImg(double dKelvin);
    double objSigToPow(double dObjSig);
    USHORT powToImg(double dPow);
    void   UpdateDeviceOP();

    CTemperature imgToTemp(long lPixval);
    CTemperature objSigToTemp(double dObjSig);

    double doCalcK1(void);
    double doCalcK2(double dAmbObjSig, double dAtmObjSig, double dExtOptTempObjSig);
    double doCalcAtmTao(void);
    void   doUpdateCalcConst(void);

public:
	afx_msg void OnCbnSelchangePalette();
	CComboBox m_ctrlPalette;
	afx_msg void OnBnClickedAutofocus();
	CButton m_btnAutoFocus;
	CButton m_btnSaveImage;
	afx_msg void OnBnClickedSaveImage();
	CComboBox m_ctrlFrameRate;
	afx_msg void OnCbnSelchangeFrate();
	CComboBox m_ctrlIRFormat;
	afx_msg void OnCbnSelchangePixelFormat();
	CComboBox m_ctrlRange;
	afx_msg void OnCbnSelchangeRange();
	afx_msg void OnTimer(UINT_PTR nIDEvent);
	CStatic m_ctrlSpot;
	CButton m_btnAutoAdjust;
	CButton m_btnAdjustOnce;
	CEdit m_ctrlScaleLow;
	CEdit m_ctrlScaleHigh;
	afx_msg void OnBnClickedAutoadj();
	float m_fScaleLo;
	CStatic m_ctrlUnit;
	float m_fScaleHi;
	afx_msg void OnBnClickedOneshotadj();
	afx_msg void OnEnKillfocusSclo();
	afx_msg void OnEnKillfocusSchi();
	afx_msg void OnBnClickedNuc();
	CButton m_btnNUC;
	afx_msg void OnBnClickedAlarmSetup();
	CColorButton m_ctrlAlarmState;
	CStatic m_ctrlTrigCount2;
	CStatic m_ctrlTrigCount1;
	CStatic m_ctrlAreaAvg;
	CStatic m_ctrlAreaMin;
	CStatic m_ctrlAreaMax;
	afx_msg void OnBnClickedAreaSetup();
	CButton m_ctrlAreaSetup;
	CButton m_btnSaveBMP;
	afx_msg void OnBnClickedSaveBmp();
	CButton m_btnDisplay;
	afx_msg void OnBnClickedAbout();
	afx_msg void OnBnClickedLog();
	CButton m_btnLog;
    CStatic m_ctrlCenterSpot;
    afx_msg void OnBnClickedCalibrate();
    CButton m_ctrlCalibrate;
    CStatic m_ctrlStatus;
    CComboBox m_ctrlPresentation;
    afx_msg void OnCbnSelchangePresentation();
    CButton m_btnSaveRAW;
    afx_msg void OnBnClickedSaveRaw();
    CStatic m_ctrlPort1State;
    CStatic m_ctrlPort2State;
    CButton m_ctrlObjPars;
    afx_msg void OnBnClickedObjpars();
    afx_msg void OnBnClickedSaveSeq();
    CColorButton m_btnSaveSEQ;
    UINT m_uiSeqFrames;
};

